@@ -353,7 +353,7 @@ G_DEFINE_TYPE (GstDiscovererInfo, gst_discoverer_info, G_TYPE_OBJECT);
 static void
 gst_discoverer_info_init (GstDiscovererInfo * info)
 {
-  /* Nothing needs initialization */
+  info->missing_elements_details = g_ptr_array_new_with_free_func (g_free);
 }
 
 static void
@@ -375,6 +375,8 @@ gst_discoverer_info_finalize (GObject * object)
 
   if (info->toc)
     gst_toc_unref (info->toc);
+
+  g_ptr_array_unref (info->missing_elements_details);
 }
 
 static GstDiscovererInfo *
@@ -687,6 +689,9 @@ gst_discoverer_stream_info_get_stream_id (GstDiscovererStreamInfo * info)
  * gst_discoverer_stream_info_get_misc:
  * @info: a #GstDiscovererStreamInfo
  *
+ * Deprecated: This functions is deprecated since version 1.4, use
+ * #gst_discoverer_info_get_missing_elements_installer_details
+ *
  * Returns: (transfer none): additional information regarding the stream (for
  * example codec version, profile, etc..). If you wish to use the #GstStructure
  * after the life-time of @info you will need to copy it.
@@ -1010,6 +1015,9 @@ DISCOVERER_INFO_ACCESSOR_CODE (seekable, gboolean, FALSE);
  * gst_discoverer_info_get_misc:
  * @info: a #GstDiscovererInfo
  *
+ * Deprecated: This functions is deprecated since version 1.4, use
+ * #gst_discoverer_info_get_missing_elements_installer_details
+ *
  * Returns: (transfer none): Miscellaneous information stored as a #GstStructure
  * (for example: information about missing plugins). If you wish to use the
  * #GstStructure after the life-time of @info, you will need to copy it.
@@ -1068,3 +1076,39 @@ DISCOVERER_INFO_ACCESSOR_CODE (toc, const GstToc *, NULL);
  *
  * Decrements the reference count of @info.
  */
+
+
+/**
+ * gst_discoverer_info_get_missing_elements_installer_details:
+ * @info: a #GstDiscovererStreamInfo to retrieve installer detail
+ * for the missing element
+ *
+ * Get the installer details for missing elements
+ *
+ * Returns: (transfer none) (array zero-terminated=1): An array of strings
+ * containing informations about how to install the various missing elements
+ * for @info to be usable. If you wish to use the strings after the life-time
+ * of @info, you will need to copy them.
+ *
+ * Since: 1.4
+ */
+const gchar **
+gst_discoverer_info_get_missing_elements_installer_details (const
+    GstDiscovererInfo * info)
+{
+
+  if (info->result != GST_DISCOVERER_MISSING_PLUGINS) {
+    GST_WARNING_OBJECT (info, "Trying to get missing element installed details "
+        "but result is not 'MISSING_PLUGINS'");
+
+    return NULL;
+  }
+
+  if (info->missing_elements_details->pdata[info->missing_elements_details->
+          len]) {
+    GST_DEBUG ("Adding NULL pointer to the end of missing_elements_details");
+    g_ptr_array_add (info->missing_elements_details, NULL);
+  }
+
+  return (const gchar **) info->missing_elements_details->pdata;
+}
