@@ -141,6 +141,10 @@ gboolean      gst_install_plugins_installation_in_progress (void);
 
 gboolean      gst_install_plugins_supported (void);
 
+#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstInstallPluginsContext, gst_install_plugins_context_free)
+#endif
+
 G_END_DECLS
 
 #endif /* __GST_PB_UTILS_INSTALL_PLUGINS_H__ */
