@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -144,9 +144,9 @@ copy_and_clean_caps (const GstCaps * caps)
   /* rtp fields */
   gst_structure_remove_field (s, "config");
   gst_structure_remove_field (s, "clock-rate");
-  gst_structure_remove_field (s, "clock-base");
+  gst_structure_remove_field (s, "timestamp-offset");
   gst_structure_remove_field (s, "maxps");
-  gst_structure_remove_field (s, "seqnum-base");
+  gst_structure_remove_field (s, "seqnum-offset");
   gst_structure_remove_field (s, "npt-start");
   gst_structure_remove_field (s, "npt-stop");
   gst_structure_remove_field (s, "play-speed");
@@ -406,23 +406,25 @@ gst_missing_plugin_message_get_installer_detail (GstMessage * msg)
   GString *str = NULL;
   gchar *detail = NULL;
   gchar *desc;
+  const GstStructure *structure;
 
   g_return_val_if_fail (gst_is_missing_plugin_message (msg), NULL);
 
-  GST_LOG ("Parsing missing-plugin message: %" GST_PTR_FORMAT, msg->structure);
+  structure = gst_message_get_structure (msg);
+  GST_LOG ("Parsing missing-plugin message: %" GST_PTR_FORMAT, structure);
 
-  missing_type = missing_structure_get_type (msg->structure);
+  missing_type = missing_structure_get_type (structure);
   if (missing_type == GST_MISSING_TYPE_UNKNOWN) {
     GST_WARNING ("couldn't parse 'type' field");
     goto error;
   }
 
-  type = gst_structure_get_string (msg->structure, "type");
+  type = gst_structure_get_string (structure, "type");
   g_assert (type != NULL);      /* validity already checked above */
 
   /* FIXME: use gst_installer_detail_new() here too */
   str = g_string_new (GST_DETAIL_STRING_MARKER "|");
-  g_string_append_printf (str, "%u.%u|", GST_VERSION_MAJOR, GST_VERSION_MINOR);
+  g_string_append_printf (str, "%s|", GST_API_VERSION);
 
   progname = (const gchar *) g_get_prgname ();
   if (progname) {
@@ -444,14 +446,14 @@ gst_missing_plugin_message_get_installer_detail (GstMessage * msg)
     case GST_MISSING_TYPE_URISOURCE:
     case GST_MISSING_TYPE_URISINK:
     case GST_MISSING_TYPE_ELEMENT:
-      if (!missing_structure_get_string_detail (msg->structure, &detail))
+      if (!missing_structure_get_string_detail (structure, &detail))
         goto error;
       break;
     case GST_MISSING_TYPE_DECODER:
     case GST_MISSING_TYPE_ENCODER:{
       GstCaps *caps = NULL;
 
-      if (!missing_structure_get_caps_detail (msg->structure, &caps))
+      if (!missing_structure_get_caps_detail (structure, &caps))
         goto error;
 
       detail = gst_caps_to_string (caps);
@@ -498,19 +500,21 @@ gst_missing_plugin_message_get_description (GstMessage * msg)
   GstMissingType missing_type;
   const gchar *desc;
   gchar *ret = NULL;
+  const GstStructure *structure;
 
   g_return_val_if_fail (gst_is_missing_plugin_message (msg), NULL);
 
-  GST_LOG ("Parsing missing-plugin message: %" GST_PTR_FORMAT, msg->structure);
+  structure = gst_message_get_structure (msg);
+  GST_LOG ("Parsing missing-plugin message: %" GST_PTR_FORMAT, structure);
 
-  desc = gst_structure_get_string (msg->structure, "name");
+  desc = gst_structure_get_string (structure, "name");
   if (desc != NULL && *desc != '\0') {
     ret = g_strdup (desc);
     goto done;
   }
 
   /* fallback #1 */
-  missing_type = missing_structure_get_type (msg->structure);
+  missing_type = missing_structure_get_type (structure);
 
   switch (missing_type) {
     case GST_MISSING_TYPE_URISOURCE:
@@ -518,13 +522,13 @@ gst_missing_plugin_message_get_description (GstMessage * msg)
     case GST_MISSING_TYPE_ELEMENT:{
       gchar *detail = NULL;
 
-      if (missing_structure_get_string_detail (msg->structure, &detail)) {
+      if (missing_structure_get_string_detail (structure, &detail)) {
         if (missing_type == GST_MISSING_TYPE_URISOURCE)
           ret = gst_pb_utils_get_source_description (detail);
         else if (missing_type == GST_MISSING_TYPE_URISINK)
           ret = gst_pb_utils_get_sink_description (detail);
         else
-          ret = gst_pb_utils_get_sink_description (detail);
+          ret = gst_pb_utils_get_element_description (detail);
         g_free (detail);
       }
       break;
@@ -533,7 +537,7 @@ gst_missing_plugin_message_get_description (GstMessage * msg)
     case GST_MISSING_TYPE_ENCODER:{
       GstCaps *caps = NULL;
 
-      if (missing_structure_get_caps_detail (msg->structure, &caps)) {
+      if (missing_structure_get_caps_detail (structure, &caps)) {
         if (missing_type == GST_MISSING_TYPE_DECODER)
           ret = gst_pb_utils_get_decoder_description (caps);
         else
@@ -591,13 +595,16 @@ done:
 gboolean
 gst_is_missing_plugin_message (GstMessage * msg)
 {
+  const GstStructure *structure;
+
   g_return_val_if_fail (msg != NULL, FALSE);
   g_return_val_if_fail (GST_IS_MESSAGE (msg), FALSE);
 
-  if (GST_MESSAGE_TYPE (msg) != GST_MESSAGE_ELEMENT || msg->structure == NULL)
+  structure = gst_message_get_structure (msg);
+  if (GST_MESSAGE_TYPE (msg) != GST_MESSAGE_ELEMENT || structure == NULL)
     return FALSE;
 
-  return gst_structure_has_name (msg->structure, "missing-plugin");
+  return gst_structure_has_name (structure, "missing-plugin");
 }
 
 /* takes ownership of the description */
@@ -609,7 +616,7 @@ gst_installer_detail_new (gchar * description, const gchar * type,
   GString *s;
 
   s = g_string_new (GST_DETAIL_STRING_MARKER "|");
-  g_string_append_printf (s, "%u.%u|", GST_VERSION_MAJOR, GST_VERSION_MINOR);
+  g_string_append_printf (s, "%s|", GST_API_VERSION);
 
   progname = (const gchar *) g_get_prgname ();
   if (progname) {
@@ -647,8 +654,6 @@ gst_installer_detail_new (gchar * description, const gchar * type,
  *
  * Returns: a newly-allocated detail string, or NULL on error. Free string
  *          with g_free() when not needed any longer.
- *
- * Since: 0.10.15
  */
 gchar *
 gst_missing_uri_source_installer_detail_new (const gchar * protocol)
@@ -677,8 +682,6 @@ gst_missing_uri_source_installer_detail_new (const gchar * protocol)
  *
  * Returns: a newly-allocated detail string, or NULL on error. Free string
  *          with g_free() when not needed any longer.
- *
- * Since: 0.10.15
  */
 gchar *
 gst_missing_uri_sink_installer_detail_new (const gchar * protocol)
@@ -707,8 +710,6 @@ gst_missing_uri_sink_installer_detail_new (const gchar * protocol)
  *
  * Returns: a newly-allocated detail string, or NULL on error. Free string
  *          with g_free() when not needed any longer.
- *
- * Since: 0.10.15
  */
 gchar *
 gst_missing_element_installer_detail_new (const gchar * factory_name)
@@ -736,8 +737,6 @@ gst_missing_element_installer_detail_new (const gchar * factory_name)
  *
  * Returns: a newly-allocated detail string, or NULL on error. Free string
  *          with g_free() when not needed any longer.
- *
- * Since: 0.10.15
  */
 gchar *
 gst_missing_decoder_installer_detail_new (const GstCaps * decode_caps)
@@ -776,8 +775,6 @@ gst_missing_decoder_installer_detail_new (const GstCaps * decode_caps)
  *
  * Returns: a newly-allocated detail string, or NULL on error. Free string
  *          with g_free() when not needed any longer.
- *
- * Since: 0.10.15
  */
 gchar *
 gst_missing_encoder_installer_detail_new (const GstCaps * encode_caps)
