@@ -59,10 +59,10 @@ gst_riff_create_video_caps (guint32 codec_fcc,
   GST_DEBUG ("video fourcc %" GST_FOURCC_FORMAT, GST_FOURCC_ARGS (codec_fcc));
 
   switch (codec_fcc) {
-    case GST_MAKE_FOURCC ('D', 'I', 'B', ' '): /* uncompressed RGB */
-    case GST_MAKE_FOURCC (0x00, 0x00, 0x00, 0x00):
-    case GST_MAKE_FOURCC ('R', 'G', 'B', ' '):
-    case GST_MAKE_FOURCC ('R', 'A', 'W', ' '):
+    case GST_RIFF_DIB:         /* uncompressed RGB */
+    case GST_RIFF_rgb:
+    case GST_RIFF_RGB:
+    case GST_RIFF_RAW:
     {
       gint bpp = (strf && strf->bit_cnt != 0) ? strf->bit_cnt : 8;
 
@@ -114,22 +114,26 @@ gst_riff_create_video_caps (guint32 codec_fcc,
         *codec_name = g_strdup ("Uncompressed packed RGB 10-bit 4:4:4");
       break;
 
-    case GST_MAKE_FOURCC ('I', '4', '2', '0'):
+    case GST_RIFF_I420:
+    case GST_RIFF_i420:
+    case GST_RIFF_IYUV:
       caps = gst_caps_new_simple ("video/x-raw",
           "format", G_TYPE_STRING, "I420", NULL);
       if (codec_name)
         *codec_name = g_strdup ("Uncompressed planar YUV 4:2:0");
       break;
 
-    case GST_MAKE_FOURCC ('Y', 'U', 'Y', '2'):
+    case GST_RIFF_YUY2:
+    case GST_RIFF_yuy2:
     case GST_MAKE_FOURCC ('Y', 'U', 'N', 'V'):
+    case GST_MAKE_FOURCC ('Y', 'U', 'Y', 'V'):
       caps = gst_caps_new_simple ("video/x-raw",
           "format", G_TYPE_STRING, "YUY2", NULL);
       if (codec_name)
         *codec_name = g_strdup ("Uncompressed packed YUV 4:2:2");
       break;
 
-    case GST_MAKE_FOURCC ('Y', 'V', 'U', '9'):
+    case GST_RIFF_YVU9:
       caps = gst_caps_new_simple ("video/x-raw",
           "format", G_TYPE_STRING, "YVU9", NULL);
       if (codec_name)
@@ -138,13 +142,15 @@ gst_riff_create_video_caps (guint32 codec_fcc,
 
     case GST_MAKE_FOURCC ('U', 'Y', 'V', 'Y'):
     case GST_MAKE_FOURCC ('2', 'v', 'u', 'y'):
+    case GST_MAKE_FOURCC ('H', 'D', 'Y', 'C'):
       caps = gst_caps_new_simple ("video/x-raw",
           "format", G_TYPE_STRING, "UYVY", NULL);
       if (codec_name)
         *codec_name = g_strdup ("Uncompressed packed YUV 4:2:2");
       break;
 
-    case GST_MAKE_FOURCC ('Y', 'V', '1', '2'):
+    case GST_RIFF_YV12:
+    case GST_RIFF_yv12:
       caps = gst_caps_new_simple ("video/x-raw",
           "format", G_TYPE_STRING, "YV12", NULL);
       if (codec_name)
@@ -157,11 +163,13 @@ gst_riff_create_video_caps (guint32 codec_fcc,
         *codec_name = g_strdup ("Uncompressed packed 10-bit YUV 4:2:2");
       break;
 
-    case GST_MAKE_FOURCC ('M', 'J', 'P', 'G'): /* YUY2 MJPEG */
+    case GST_RIFF_MJPG:        /* YUY2 MJPEG */
+    case GST_RIFF_mJPG:
     case GST_MAKE_FOURCC ('A', 'V', 'R', 'n'):
-    case GST_MAKE_FOURCC ('I', 'J', 'P', 'G'):
+    case GST_RIFF_IJPG:
     case GST_MAKE_FOURCC ('i', 'j', 'p', 'g'):
-    case GST_MAKE_FOURCC ('d', 'm', 'b', '1'):
+    case GST_RIFF_DMB1:
+    case GST_RIFF_dmb1:
     case GST_MAKE_FOURCC ('A', 'C', 'D', 'V'):
     case GST_MAKE_FOURCC ('Q', 'I', 'V', 'G'):
       caps = gst_caps_new_empty_simple ("image/jpeg");
@@ -169,7 +177,8 @@ gst_riff_create_video_caps (guint32 codec_fcc,
         *codec_name = g_strdup ("Motion JPEG");
       break;
 
-    case GST_MAKE_FOURCC ('J', 'P', 'E', 'G'): /* generic (mostly RGB) MJPEG */
+    case GST_RIFF_JPEG:        /* generic (mostly RGB) MJPEG */
+    case GST_RIFF_jpeg:
     case GST_MAKE_FOURCC ('j', 'p', 'e', 'g'): /* generic (mostly RGB) MJPEG */
       caps = gst_caps_new_empty_simple ("image/jpeg");
       if (codec_name)
@@ -177,7 +186,8 @@ gst_riff_create_video_caps (guint32 codec_fcc,
       break;
 
     case GST_MAKE_FOURCC ('P', 'I', 'X', 'L'): /* Miro/Pinnacle fourccs */
-    case GST_MAKE_FOURCC ('V', 'I', 'X', 'L'): /* Miro/Pinnacle fourccs */
+    case GST_RIFF_VIXL:        /* Miro/Pinnacle fourccs */
+    case GST_RIFF_vixl:
       caps = gst_caps_new_empty_simple ("image/jpeg");
       if (codec_name)
         *codec_name = g_strdup ("Miro/Pinnacle Motion JPEG");
@@ -267,9 +277,9 @@ gst_riff_create_video_caps (guint32 codec_fcc,
         *codec_name = g_strdup ("Lead MPEG-2 video");
       break;
 
-    case GST_MAKE_FOURCC ('H', '2', '6', '3'):
-    case GST_MAKE_FOURCC ('h', '2', '6', '3'):
-    case GST_MAKE_FOURCC ('i', '2', '6', '3'):
+    case GST_RIFF_H263:
+    case GST_RIFF_h263:
+    case GST_RIFF_i263:
     case GST_MAKE_FOURCC ('U', '2', '6', '3'):
     case GST_MAKE_FOURCC ('v', 'i', 'v', '1'):
     case GST_MAKE_FOURCC ('T', '2', '6', '3'):
@@ -279,7 +289,7 @@ gst_riff_create_video_caps (guint32 codec_fcc,
         *codec_name = g_strdup ("ITU H.26n");
       break;
 
-    case GST_MAKE_FOURCC ('L', '2', '6', '3'):
+    case GST_RIFF_L263:
       /* http://www.leadcodecs.com/Codecs/LEAD-H263.htm */
       caps = gst_caps_new_simple ("video/x-h263",
           "variant", G_TYPE_STRING, "lead", NULL);
@@ -287,15 +297,15 @@ gst_riff_create_video_caps (guint32 codec_fcc,
         *codec_name = g_strdup ("Lead H.263");
       break;
 
-    case GST_MAKE_FOURCC ('M', '2', '6', '3'):
-    case GST_MAKE_FOURCC ('m', '2', '6', '3'):
+    case GST_RIFF_M263:
+    case GST_RIFF_m263:
       caps = gst_caps_new_simple ("video/x-h263",
           "variant", G_TYPE_STRING, "microsoft", NULL);
       if (codec_name)
         *codec_name = g_strdup ("Microsoft H.263");
       break;
 
-    case GST_MAKE_FOURCC ('V', 'D', 'O', 'W'):
+    case GST_RIFF_VDOW:
       caps = gst_caps_new_simple ("video/x-h263",
           "variant", G_TYPE_STRING, "vdolive", NULL);
       if (codec_name)
@@ -309,7 +319,7 @@ gst_riff_create_video_caps (guint32 codec_fcc,
         *codec_name = g_strdup ("Vivo H.263");
       break;
 
-    case GST_MAKE_FOURCC ('x', '2', '6', '3'):
+    case GST_RIFF_x263:
       caps = gst_caps_new_simple ("video/x-h263",
           "variant", G_TYPE_STRING, "xirlink", NULL);
       if (codec_name)
@@ -343,7 +353,7 @@ gst_riff_create_video_caps (guint32 codec_fcc,
         *codec_name = g_strdup ("ITU H.264");
       break;
 
-    case GST_MAKE_FOURCC ('V', 'S', 'S', 'H'):
+    case GST_RIFF_VSSH:
       caps = gst_caps_new_simple ("video/x-h264",
           "variant", G_TYPE_STRING, "videosoft", NULL);
       if (codec_name)
@@ -374,7 +384,7 @@ gst_riff_create_video_caps (guint32 codec_fcc,
         *codec_name = g_strdup ("Divio MPEG-4");
       break;
 
-    case GST_MAKE_FOURCC ('D', 'I', 'V', '3'):
+    case GST_RIFF_DIV3:
     case GST_MAKE_FOURCC ('d', 'i', 'v', '3'):
     case GST_MAKE_FOURCC ('D', 'V', 'X', '3'):
     case GST_MAKE_FOURCC ('d', 'v', 'x', '3'):
@@ -577,18 +587,19 @@ gst_riff_create_video_caps (guint32 codec_fcc,
         *codec_name = g_strdup ("Microsoft Windows Media VC-1");
       break;
 
-    case GST_MAKE_FOURCC ('c', 'v', 'i', 'd'):
+    case GST_RIFF_cvid:
+    case GST_RIFF_CVID:
       caps = gst_caps_new_empty_simple ("video/x-cinepak");
       if (codec_name)
         *codec_name = g_strdup ("Cinepak video");
       break;
 
-    case GST_MAKE_FOURCC ('M', 'S', 'V', 'C'):
-    case GST_MAKE_FOURCC ('m', 's', 'v', 'c'):
-    case GST_MAKE_FOURCC ('C', 'R', 'A', 'M'):
-    case GST_MAKE_FOURCC ('c', 'r', 'a', 'm'):
-    case GST_MAKE_FOURCC ('W', 'H', 'A', 'M'):
-    case GST_MAKE_FOURCC ('w', 'h', 'a', 'm'):
+    case GST_RIFF_FCCH_MSVC:
+    case GST_RIFF_FCCH_msvc:
+    case GST_RIFF_CRAM:
+    case GST_RIFF_cram:
+    case GST_RIFF_WHAM:
+    case GST_RIFF_wham:
       caps = gst_caps_new_simple ("video/x-msvideocodec",
           "msvideoversion", G_TYPE_INT, 1, NULL);
       if (strf) {
@@ -601,7 +612,7 @@ gst_riff_create_video_caps (guint32 codec_fcc,
       strf_data = NULL;
       break;
 
-    case GST_MAKE_FOURCC ('R', 'L', 'E', ' '):
+    case GST_RIFF_FCCH_RLE:
     case GST_MAKE_FOURCC ('m', 'r', 'l', 'e'):
     case GST_MAKE_FOURCC (0x1, 0x0, 0x0, 0x0): /* why, why, why? */
     case GST_MAKE_FOURCC (0x2, 0x0, 0x0, 0x0): /* why, why, why? */
@@ -632,32 +643,33 @@ gst_riff_create_video_caps (guint32 codec_fcc,
         *codec_name = g_strdup ("Xan Wing Commander 4");
       break;
 
-    case GST_MAKE_FOURCC ('R', 'T', '2', '1'):
+    case GST_RIFF_RT21:
+    case GST_RIFF_rt21:
       caps = gst_caps_new_simple ("video/x-indeo",
           "indeoversion", G_TYPE_INT, 2, NULL);
       if (codec_name)
         *codec_name = g_strdup ("Intel Video 2");
       break;
 
-    case GST_MAKE_FOURCC ('I', 'V', '3', '1'):
-    case GST_MAKE_FOURCC ('I', 'V', '3', '2'):
-    case GST_MAKE_FOURCC ('i', 'v', '3', '1'):
-    case GST_MAKE_FOURCC ('i', 'v', '3', '2'):
+    case GST_RIFF_IV31:
+    case GST_RIFF_IV32:
+    case GST_RIFF_iv31:
+    case GST_RIFF_iv32:
       caps = gst_caps_new_simple ("video/x-indeo",
           "indeoversion", G_TYPE_INT, 3, NULL);
       if (codec_name)
         *codec_name = g_strdup ("Intel Video 3");
       break;
 
-    case GST_MAKE_FOURCC ('I', 'V', '4', '1'):
-    case GST_MAKE_FOURCC ('i', 'v', '4', '1'):
+    case GST_RIFF_IV41:
+    case GST_RIFF_iv41:
       caps = gst_caps_new_simple ("video/x-indeo",
           "indeoversion", G_TYPE_INT, 4, NULL);
       if (codec_name)
         *codec_name = g_strdup ("Intel Video 4");
       break;
 
-    case GST_MAKE_FOURCC ('I', 'V', '5', '0'):
+    case GST_RIFF_IV50:
       caps = gst_caps_new_simple ("video/x-indeo",
           "indeoversion", G_TYPE_INT, 5, NULL);
       if (codec_name)
@@ -683,8 +695,8 @@ gst_riff_create_video_caps (guint32 codec_fcc,
         *codec_name = g_strdup ("Cirrus Logipak AccuPak");
       break;
 
-    case GST_MAKE_FOURCC ('C', 'Y', 'U', 'V'):
-    case GST_MAKE_FOURCC ('c', 'y', 'u', 'v'):
+    case GST_RIFF_CYUV:
+    case GST_RIFF_cyuv:
       caps = gst_caps_new_empty_simple ("video/x-compressed-yuv");
       if (codec_name)
         *codec_name = g_strdup ("CYUV Lossless");
@@ -715,7 +727,8 @@ gst_riff_create_video_caps (guint32 codec_fcc,
         *codec_name = g_strdup ("VP3");
       break;
 
-    case GST_MAKE_FOURCC ('U', 'L', 'T', 'I'):
+    case GST_RIFF_ULTI:
+    case GST_RIFF_ulti:
       caps = gst_caps_new_empty_simple ("video/x-ultimotion");
       if (codec_name)
         *codec_name = g_strdup ("IBM UltiMotion");
@@ -904,6 +917,7 @@ gst_riff_create_video_caps (guint32 codec_fcc,
       break;
 
     case GST_MAKE_FOURCC ('D', 'X', 'S', 'B'):
+    case GST_MAKE_FOURCC ('D', 'X', 'S', 'A'):
       caps = gst_caps_new_empty_simple ("subpicture/x-xsub");
       if (codec_name)
         *codec_name = g_strdup ("XSUB subpicture stream");
@@ -1021,26 +1035,24 @@ gst_riff_wavext_add_channel_mask (GstCaps * caps, gint num_channels,
 {
   gint i, p;
   guint64 channel_mask = 0;
-  GstAudioChannelPosition from[18], to[18];
+  GstAudioChannelPosition *from, *to;
+  gboolean ret = FALSE;
 
-  if (num_channels < 1 || num_channels > MAX_CHANNEL_POSITIONS) {
+  if (num_channels < 1) {
     GST_DEBUG ("invalid number of channels: %d", num_channels);
     return FALSE;
   }
 
+  from = g_new (GstAudioChannelPosition, num_channels);
+  to = g_new (GstAudioChannelPosition, num_channels);
   p = 0;
   for (i = 0; i < MAX_CHANNEL_POSITIONS; ++i) {
     if ((layout & layout_mapping[i].ms_mask) != 0) {
       if (p >= num_channels) {
         GST_WARNING ("More bits set in the channel layout map than there "
-            "are channels! Broken file");
-        return FALSE;
-      }
-      if (layout_mapping[i].gst_pos == GST_AUDIO_CHANNEL_POSITION_INVALID) {
-        GST_WARNING ("Unsupported channel position (mask 0x%08x) in channel "
-            "layout map - ignoring those channels", layout_mapping[i].ms_mask);
-        /* what to do? just ignore it and let downstream deal with a channel
-         * layout that has INVALID positions in it for now ... */
+            "are channels! Setting channel-mask to 0.");
+        channel_mask = 0;
+        break;
       }
       channel_mask |= G_GUINT64_CONSTANT (1) << layout_mapping[i].gst_pos;
       from[p] = layout_mapping[i].gst_pos;
@@ -1048,25 +1060,33 @@ gst_riff_wavext_add_channel_mask (GstCaps * caps, gint num_channels,
     }
   }
 
-  if (p != num_channels) {
-    GST_WARNING ("Only %d bits set in the channel layout map, but there are "
-        "supposed to be %d channels! Broken file", p, num_channels);
-    return FALSE;
-  }
-
-  if (channel_reorder_map) {
+  if (channel_mask > 0 && channel_reorder_map) {
+    if (p != num_channels) {
+      /* WAVEFORMATEXTENSIBLE allows to have more channels than bits in
+       * the channel mask. We accept this, too, and hope that downstream
+       * can handle this */
+      GST_WARNING ("Partially unknown positions in channel mask");
+      for (; p < num_channels; ++p)
+        from[p] = GST_AUDIO_CHANNEL_POSITION_INVALID;
+    }
     memcpy (to, from, sizeof (from[0]) * num_channels);
     if (!gst_audio_channel_positions_to_valid_order (to, num_channels))
-      return FALSE;
+      goto fail;
     if (!gst_audio_get_channel_reorder_map (num_channels, from, to,
             channel_reorder_map))
-      return FALSE;
+      goto fail;
   }
 
   gst_caps_set_simple (caps, "channel-mask", GST_TYPE_BITMASK, channel_mask,
       NULL);
 
-  return TRUE;
+  ret = TRUE;
+
+fail:
+  g_free (from);
+  g_free (to);
+
+  return ret;
 }
 
 static gboolean
@@ -1202,17 +1222,21 @@ gst_riff_create_audio_caps (guint16 codec_id,
         gint wd, ws;
         GstAudioFormat format;
 
-        /* If we have an empty blockalign, we take the width contained in 
-         * strf->bits_per_sample */
-        if (ba != 0)
+        if (ba > (32 / 8) * ch) {
+          GST_WARNING ("Invalid block align: %d > %d", ba, (32 / 8) * ch);
+          wd = GST_ROUND_UP_8 (strf->bits_per_sample);
+        } else if (ba != 0) {
+          /* If we have an empty blockalign, we take the width contained in 
+           * strf->bits_per_sample */
           wd = ba * 8 / ch;
-        else
-          wd = strf->bits_per_sample;
+        } else {
+          wd = GST_ROUND_UP_8 (strf->bits_per_sample);
+        }
 
         if (strf->bits_per_sample > 32) {
           GST_WARNING ("invalid depth (%d) of pcm audio, overwriting.",
               strf->bits_per_sample);
-          strf->bits_per_sample = 8 * ((wd + 7) / 8);
+          strf->bits_per_sample = wd;
         }
 
         /* in riff, the depth is stored in the size field but it just means that
@@ -1223,6 +1247,10 @@ gst_riff_create_audio_caps (guint16 codec_id,
 
         format =
             gst_audio_format_build_integer (wd != 8, G_LITTLE_ENDIAN, wd, ws);
+        if (format == GST_AUDIO_FORMAT_UNKNOWN) {
+          GST_WARNING ("Unsupported raw audio format with width %d", wd);
+          return NULL;
+        }
 
         caps = gst_caps_new_simple ("audio/x-raw",
             "format", G_TYPE_STRING, gst_audio_format_to_string (format),
