@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -39,15 +39,21 @@ struct _GstRTPBaseDepayloadPrivate
   GstClockTime npt_stop;
   gdouble play_speed;
   gdouble play_scale;
+  guint clock_base;
 
   gboolean discont;
   GstClockTime pts;
   GstClockTime dts;
   GstClockTime duration;
 
+  guint32 last_seqnum;
+  guint32 last_rtptime;
   guint32 next_seqnum;
 
   gboolean negotiated;
+
+  GstCaps *last_caps;
+  GstEvent *segment_event;
 };
 
 /* Filter signals and args */
@@ -60,6 +66,7 @@ enum
 enum
 {
   PROP_0,
+  PROP_STATS,
   PROP_LAST
 };
 
@@ -71,6 +78,8 @@ static void gst_rtp_base_depayload_get_property (GObject * object,
 
 static GstFlowReturn gst_rtp_base_depayload_chain (GstPad * pad,
     GstObject * parent, GstBuffer * in);
+static GstFlowReturn gst_rtp_base_depayload_chain_list (GstPad * pad,
+    GstObject * parent, GstBufferList * list);
 static gboolean gst_rtp_base_depayload_handle_sink_event (GstPad * pad,
     GstObject * parent, GstEvent * event);
 
@@ -87,6 +96,8 @@ static void gst_rtp_base_depayload_class_init (GstRTPBaseDepayloadClass *
     klass);
 static void gst_rtp_base_depayload_init (GstRTPBaseDepayload * rtpbasepayload,
     GstRTPBaseDepayloadClass * klass);
+static GstEvent *create_segment_event (GstRTPBaseDepayload * filter,
+    guint rtptime, GstClockTime position);
 
 GType
 gst_rtp_base_depayload_get_type (void)
@@ -129,6 +140,69 @@ gst_rtp_base_depayload_class_init (GstRTPBaseDepayloadClass * klass)
   gobject_class->set_property = gst_rtp_base_depayload_set_property;
   gobject_class->get_property = gst_rtp_base_depayload_get_property;
 
+
+  /**
+   * GstRTPBaseDepayload:stats:
+   *
+   * Various depayloader statistics retrieved atomically (and are therefore
+   * synchroized with each other). This property return a GstStructure named
+   * application/x-rtp-depayload-stats containing the following fields relating to
+   * the last processed buffer and current state of the stream being depayloaded:
+   *
+   * <variablelist>
+   *   <varlistentry>
+   *     <term>clock-rate</term>
+   *     <listitem><para>#G_TYPE_UINT, clock-rate of the
+   *     stream</para></listitem>
+   *   </varlistentry>
+   *   <varlistentry>
+   *     <term>npt-start</term>
+   *     <listitem><para>#G_TYPE_UINT64, time of playback start
+   *     </para></listitem>
+   *   </varlistentry>
+   *   <varlistentry>
+   *     <term>npt-stop</term>
+   *     <listitem><para>#G_TYPE_UINT64, time of playback stop
+   *     </para></listitem>
+   *   </varlistentry>
+   *   <varlistentry>
+   *     <term>play-speed</term>
+   *     <listitem><para>#G_TYPE_DOUBLE, the playback speed
+   *     </para></listitem>
+   *   </varlistentry>
+   *   <varlistentry>
+   *     <term>play-scale</term>
+   *     <listitem><para>#G_TYPE_DOUBLE, the playback scale
+   *     </para></listitem>
+   *   </varlistentry>
+   *   <varlistentry>
+   *     <term>running-time-dts</term>
+   *     <listitem><para>#G_TYPE_UINT64, the last running-time of the
+   *      last DTS
+   *     </para></listitem>
+   *   </varlistentry>
+   *   <varlistentry>
+   *     <term>running-time-pts</term>
+   *     <listitem><para>#G_TYPE_UINT64, the last running-time of the
+   *      last PTS
+   *     </para></listitem>
+   *   </varlistentry>
+   *   <varlistentry>
+   *     <term>seqnum</term>
+   *     <listitem><para>#G_TYPE_UINT, the last seen seqnum
+   *     </para></listitem>
+   *   </varlistentry>
+   *   <varlistentry>
+   *     <term>timestamp</term>
+   *     <listitem><para>#G_TYPE_UINT, the last seen RTP timestamp
+   *     </para></listitem>
+   *   </varlistentry>
+   * </variablelist>
+   **/
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_STATS,
+      g_param_spec_boxed ("stats", "Statistics", "Various statistics",
+          GST_TYPE_STRUCTURE, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
   gstelement_class->change_state = gst_rtp_base_depayload_change_state;
 
   klass->packet_lost = gst_rtp_base_depayload_packet_lost;
@@ -155,6 +229,8 @@ gst_rtp_base_depayload_init (GstRTPBaseDepayload * filter,
   g_return_if_fail (pad_template != NULL);
   filter->sinkpad = gst_pad_new_from_template (pad_template, "sink");
   gst_pad_set_chain_function (filter->sinkpad, gst_rtp_base_depayload_chain);
+  gst_pad_set_chain_list_function (filter->sinkpad,
+      gst_rtp_base_depayload_chain_list);
   gst_pad_set_event_function (filter->sinkpad,
       gst_rtp_base_depayload_handle_sink_event);
   gst_element_add_pad (GST_ELEMENT (filter), filter->sinkpad);
@@ -166,6 +242,15 @@ gst_rtp_base_depayload_init (GstRTPBaseDepayload * filter,
   gst_pad_use_fixed_caps (filter->srcpad);
   gst_element_add_pad (GST_ELEMENT (filter), filter->srcpad);
 
+  priv->npt_start = 0;
+  priv->npt_stop = -1;
+  priv->play_speed = 1.0;
+  priv->play_scale = 1.0;
+  priv->clock_base = -1;
+  priv->dts = -1;
+  priv->pts = -1;
+  priv->duration = -1;
+
   gst_segment_init (&filter->segment, GST_FORMAT_UNDEFINED);
 }
 
@@ -188,7 +273,17 @@ gst_rtp_base_depayload_setcaps (GstRTPBaseDepayload * filter, GstCaps * caps)
 
   bclass = GST_RTP_BASE_DEPAYLOAD_GET_CLASS (filter);
 
-  GST_DEBUG_OBJECT (filter, "Set caps");
+  GST_DEBUG_OBJECT (filter, "Set caps %" GST_PTR_FORMAT, caps);
+
+  if (priv->last_caps) {
+    if (gst_caps_is_equal (priv->last_caps, caps)) {
+      res = TRUE;
+      goto caps_not_changed;
+    } else {
+      gst_caps_unref (priv->last_caps);
+      priv->last_caps = NULL;
+    }
+  }
 
   caps_struct = gst_caps_get_structure (caps, 0);
 
@@ -220,6 +315,12 @@ gst_rtp_base_depayload_setcaps (GstRTPBaseDepayload * filter, GstCaps * caps)
   else
     priv->play_scale = 1.0;
 
+  value = gst_structure_get_value (caps_struct, "clock-base");
+  if (value && G_VALUE_HOLDS_UINT (value))
+    priv->clock_base = g_value_get_uint (value);
+  else
+    priv->clock_base = -1;
+
   if (bclass->set_caps) {
     res = bclass->set_caps (filter, caps);
     if (!res) {
@@ -232,27 +333,40 @@ gst_rtp_base_depayload_setcaps (GstRTPBaseDepayload * filter, GstCaps * caps)
 
   priv->negotiated = res;
 
+  if (priv->negotiated)
+    priv->last_caps = gst_caps_ref (caps);
+
   return res;
+
+caps_not_changed:
+  {
+    GST_DEBUG_OBJECT (filter, "Caps did not change");
+    return res;
+  }
 }
 
+/* takes ownership of the input buffer */
 static GstFlowReturn
-gst_rtp_base_depayload_chain (GstPad * pad, GstObject * parent, GstBuffer * in)
+gst_rtp_base_depayload_handle_buffer (GstRTPBaseDepayload * filter,
+    GstRTPBaseDepayloadClass * bclass, GstBuffer * in)
 {
-  GstRTPBaseDepayload *filter;
+  GstBuffer *(*process_rtp_packet_func) (GstRTPBaseDepayload * base,
+      GstRTPBuffer * rtp_buffer);
+  GstBuffer *(*process_func) (GstRTPBaseDepayload * base, GstBuffer * in);
   GstRTPBaseDepayloadPrivate *priv;
-  GstRTPBaseDepayloadClass *bclass;
   GstFlowReturn ret = GST_FLOW_OK;
   GstBuffer *out_buf;
-  GstClockTime pts, dts;
   guint16 seqnum;
   guint32 rtptime;
-  gboolean discont;
+  gboolean discont, buf_discont;
   gint gap;
   GstRTPBuffer rtp = { NULL };
 
-  filter = GST_RTP_BASE_DEPAYLOAD (parent);
   priv = filter->priv;
 
+  process_func = bclass->process;
+  process_rtp_packet_func = bclass->process_rtp_packet;
+
   /* we must have a setcaps first */
   if (G_UNLIKELY (!priv->negotiated))
     goto not_negotiated;
@@ -260,31 +374,26 @@ gst_rtp_base_depayload_chain (GstPad * pad, GstObject * parent, GstBuffer * in)
   if (G_UNLIKELY (!gst_rtp_buffer_map (in, GST_MAP_READ, &rtp)))
     goto invalid_buffer;
 
-  if (!priv->discont)
-    priv->discont = GST_BUFFER_IS_DISCONT (in);
+  buf_discont = GST_BUFFER_IS_DISCONT (in);
 
-  pts = GST_BUFFER_PTS (in);
-  dts = GST_BUFFER_DTS (in);
-  /* convert to running_time and save the timestamp, this is the timestamp
-   * we put on outgoing buffers. */
-  pts = gst_segment_to_running_time (&filter->segment, GST_FORMAT_TIME, pts);
-  dts = gst_segment_to_running_time (&filter->segment, GST_FORMAT_TIME, dts);
-  priv->pts = pts;
-  priv->dts = dts;
+  priv->pts = GST_BUFFER_PTS (in);
+  priv->dts = GST_BUFFER_DTS (in);
   priv->duration = GST_BUFFER_DURATION (in);
 
   seqnum = gst_rtp_buffer_get_seq (&rtp);
   rtptime = gst_rtp_buffer_get_timestamp (&rtp);
-  gst_rtp_buffer_unmap (&rtp);
 
-  discont = FALSE;
+  priv->last_seqnum = seqnum;
+  priv->last_rtptime = rtptime;
+
+  discont = buf_discont;
 
   GST_LOG_OBJECT (filter, "discont %d, seqnum %u, rtptime %u, pts %"
-      GST_TIME_FORMAT ", dts %" GST_TIME_FORMAT, priv->discont, seqnum, rtptime,
-      GST_TIME_ARGS (pts), GST_TIME_ARGS (dts));
+      GST_TIME_FORMAT ", dts %" GST_TIME_FORMAT, buf_discont, seqnum, rtptime,
+      GST_TIME_ARGS (priv->pts), GST_TIME_ARGS (priv->dts));
 
   /* Check seqnum. This is a very simple check that makes sure that the seqnums
-   * are striclty increasing, dropping anything that is out of the ordinary. We
+   * are strictly increasing, dropping anything that is out of the ordinary. We
    * can only do this when the next_seqnum is known. */
   if (G_LIKELY (priv->next_seqnum != -1)) {
     gap = gst_rtp_buffer_compare_seqnum (seqnum, priv->next_seqnum);
@@ -314,25 +423,49 @@ gst_rtp_base_depayload_chain (GstPad * pad, GstObject * parent, GstBuffer * in)
   }
   priv->next_seqnum = (seqnum + 1) & 0xffff;
 
-  if (G_UNLIKELY (discont && !priv->discont)) {
-    GST_LOG_OBJECT (filter, "mark DISCONT on input buffer");
-    /* we detected a seqnum discont but the buffer was not flagged with a discont,
-     * set the discont flag so that the subclass can throw away old data. */
+  if (G_UNLIKELY (discont)) {
     priv->discont = TRUE;
-    in = gst_buffer_make_writable (in);
-    GST_BUFFER_FLAG_SET (in, GST_BUFFER_FLAG_DISCONT);
+    if (!buf_discont) {
+      gpointer old_inbuf = in;
+
+      /* we detected a seqnum discont but the buffer was not flagged with a discont,
+       * set the discont flag so that the subclass can throw away old data. */
+      GST_LOG_OBJECT (filter, "mark DISCONT on input buffer");
+      in = gst_buffer_make_writable (in);
+      GST_BUFFER_FLAG_SET (in, GST_BUFFER_FLAG_DISCONT);
+      /* depayloaders will check flag on rtpbuffer->buffer, so if the input
+       * buffer was not writable already we need to remap to make our
+       * newly-flagged buffer current on the rtpbuffer */
+      if (in != old_inbuf) {
+        gst_rtp_buffer_unmap (&rtp);
+        if (G_UNLIKELY (!gst_rtp_buffer_map (in, GST_MAP_READ, &rtp)))
+          goto invalid_buffer;
+      }
+    }
   }
 
-  bclass = GST_RTP_BASE_DEPAYLOAD_GET_CLASS (filter);
+  /* prepare segment event if needed */
+  if (filter->need_newsegment) {
+    priv->segment_event = create_segment_event (filter, rtptime,
+        GST_BUFFER_PTS (in));
+    filter->need_newsegment = FALSE;
+  }
 
-  if (G_UNLIKELY (bclass->process == NULL))
+  if (process_rtp_packet_func != NULL) {
+    out_buf = process_rtp_packet_func (filter, &rtp);
+    gst_rtp_buffer_unmap (&rtp);
+  } else if (process_func != NULL) {
+    gst_rtp_buffer_unmap (&rtp);
+    out_buf = process_func (filter, in);
+  } else {
     goto no_process;
+  }
 
   /* let's send it out to processing */
-  out_buf = bclass->process (filter, in);
   if (out_buf) {
     ret = gst_rtp_base_depayload_push (filter, out_buf);
   }
+
   gst_buffer_unref (in);
 
   return ret;
@@ -362,20 +495,82 @@ invalid_buffer:
   }
 dropping:
   {
+    gst_rtp_buffer_unmap (&rtp);
     GST_WARNING_OBJECT (filter, "%d <= 100, dropping old packet", gap);
     gst_buffer_unref (in);
     return GST_FLOW_OK;
   }
 no_process:
   {
+    gst_rtp_buffer_unmap (&rtp);
     /* this is not fatal but should be filtered earlier */
     GST_ELEMENT_ERROR (filter, STREAM, NOT_IMPLEMENTED, (NULL),
-        ("The subclass does not have a process method"));
+        ("The subclass does not have a process or process_rtp_packet method"));
     gst_buffer_unref (in);
     return GST_FLOW_ERROR;
   }
 }
 
+static GstFlowReturn
+gst_rtp_base_depayload_chain (GstPad * pad, GstObject * parent, GstBuffer * in)
+{
+  GstRTPBaseDepayloadClass *bclass;
+  GstRTPBaseDepayload *basedepay;
+  GstFlowReturn flow_ret;
+
+  basedepay = GST_RTP_BASE_DEPAYLOAD_CAST (parent);
+
+  bclass = GST_RTP_BASE_DEPAYLOAD_GET_CLASS (basedepay);
+
+  flow_ret = gst_rtp_base_depayload_handle_buffer (basedepay, bclass, in);
+
+  return flow_ret;
+}
+
+static GstFlowReturn
+gst_rtp_base_depayload_chain_list (GstPad * pad, GstObject * parent,
+    GstBufferList * list)
+{
+  GstRTPBaseDepayloadClass *bclass;
+  GstRTPBaseDepayload *basedepay;
+  GstFlowReturn flow_ret;
+  GstBuffer *buffer;
+  guint i, len;
+
+  basedepay = GST_RTP_BASE_DEPAYLOAD_CAST (parent);
+
+  bclass = GST_RTP_BASE_DEPAYLOAD_GET_CLASS (basedepay);
+
+  flow_ret = GST_FLOW_OK;
+
+  /* chain each buffer in list individually */
+  len = gst_buffer_list_length (list);
+
+  if (len == 0)
+    goto done;
+
+  for (i = 0; i < len; i++) {
+    buffer = gst_buffer_list_get (list, i);
+
+    /* handle_buffer takes ownership of input buffer */
+    /* FIXME: add a way to steal buffers from list as we will unref it anyway */
+    gst_buffer_ref (buffer);
+
+    /* Should we fix up any missing timestamps for list buffers here
+     * (e.g. set to first or previous timestamp in list) or just assume
+     * the's a jitterbuffer that will have done that for us? */
+    flow_ret = gst_rtp_base_depayload_handle_buffer (basedepay, bclass, buffer);
+    if (flow_ret != GST_FLOW_OK)
+      break;
+  }
+
+done:
+
+  gst_buffer_list_unref (list);
+
+  return flow_ret;
+}
+
 static gboolean
 gst_rtp_base_depayload_handle_event (GstRTPBaseDepayload * filter,
     GstEvent * event)
@@ -385,9 +580,13 @@ gst_rtp_base_depayload_handle_event (GstRTPBaseDepayload * filter,
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_FLUSH_STOP:
+      GST_OBJECT_LOCK (filter);
       gst_segment_init (&filter->segment, GST_FORMAT_UNDEFINED);
+      GST_OBJECT_UNLOCK (filter);
+
       filter->need_newsegment = TRUE;
       filter->priv->next_seqnum = -1;
+      gst_event_replace (&filter->priv->segment_event, NULL);
       break;
     case GST_EVENT_CAPS:
     {
@@ -401,7 +600,10 @@ gst_rtp_base_depayload_handle_event (GstRTPBaseDepayload * filter,
     }
     case GST_EVENT_SEGMENT:
     {
+      GST_OBJECT_LOCK (filter);
       gst_event_copy_segment (event, &filter->segment);
+      GST_OBJECT_UNLOCK (filter);
+
       /* don't pass the event downstream, we generate our own segment including
        * the NTP time and other things we receive in caps */
       forward = FALSE;
@@ -417,7 +619,7 @@ gst_rtp_base_depayload_handle_event (GstRTPBaseDepayload * filter,
         /* we get this event from the jitterbuffer when it considers a packet as
          * being lost. We send it to our packet_lost vmethod. The default
          * implementation will make time progress by pushing out a GAP event.
-         * Subclasses can override and to one of the following:
+         * Subclasses can override and do one of the following:
          *  - Adjust timestamp/duration to something more accurate before
          *    calling the parent (default) packet_lost method.
          *  - do some more advanced error concealing on the already received
@@ -461,28 +663,63 @@ gst_rtp_base_depayload_handle_sink_event (GstPad * pad, GstObject * parent,
 }
 
 static GstEvent *
-create_segment_event (GstRTPBaseDepayload * filter, GstClockTime position)
+create_segment_event (GstRTPBaseDepayload * filter, guint rtptime,
+    GstClockTime position)
 {
   GstEvent *event;
-  GstClockTime stop;
+  GstClockTime start, stop, running_time;
   GstRTPBaseDepayloadPrivate *priv;
   GstSegment segment;
 
   priv = filter->priv;
 
+  /* We don't need the object lock around - the segment
+   * can't change here while we're holding the STREAM_LOCK
+   */
+
+  /* determining the start of the segment */
+  start = filter->segment.start;
+  if (priv->clock_base != -1 && position != -1) {
+    GstClockTime exttime, gap;
+
+    exttime = priv->clock_base;
+    gst_rtp_buffer_ext_timestamp (&exttime, rtptime);
+    gap = gst_util_uint64_scale_int (exttime - priv->clock_base,
+        filter->clock_rate, GST_SECOND);
+
+    /* account for lost packets */
+    if (position > gap) {
+      GST_DEBUG_OBJECT (filter,
+          "Found gap of %" GST_TIME_FORMAT ", adjusting start: %"
+          GST_TIME_FORMAT " = %" GST_TIME_FORMAT " - %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (gap), GST_TIME_ARGS (position - gap),
+          GST_TIME_ARGS (position), GST_TIME_ARGS (gap));
+      start = position - gap;
+    }
+  }
+
+  /* determining the stop of the segment */
+  stop = filter->segment.stop;
   if (priv->npt_stop != -1)
-    stop = priv->npt_stop - priv->npt_start;
-  else
-    stop = -1;
+    stop = start + (priv->npt_stop - priv->npt_start);
+
+  if (position == -1)
+    position = start;
+
+  running_time = gst_segment_to_running_time (&filter->segment,
+      GST_FORMAT_TIME, start);
 
   gst_segment_init (&segment, GST_FORMAT_TIME);
   segment.rate = priv->play_speed;
   segment.applied_rate = priv->play_scale;
-  segment.start = 0;
+  segment.start = start;
   segment.stop = stop;
   segment.time = priv->npt_start;
   segment.position = position;
+  segment.base = running_time;
 
+  GST_DEBUG_OBJECT (filter, "Creating segment event %" GST_SEGMENT_FORMAT,
+      &segment);
   event = gst_event_new_segment (&segment);
 
   return event;
@@ -548,14 +785,9 @@ gst_rtp_base_depayload_prepare_push (GstRTPBaseDepayload * filter,
   }
 
   /* if this is the first buffer send a NEWSEGMENT */
-  if (G_UNLIKELY (filter->need_newsegment)) {
-    GstEvent *event;
-
-    event = create_segment_event (filter, 0);
-
-    gst_pad_push_event (filter->srcpad, event);
-
-    filter->need_newsegment = FALSE;
+  if (G_UNLIKELY (filter->priv->segment_event)) {
+    gst_pad_push_event (filter->srcpad, filter->priv->segment_event);
+    filter->priv->segment_event = NULL;
     GST_DEBUG_OBJECT (filter, "Pushed newsegment event on this first buffer");
   }
 
@@ -616,13 +848,13 @@ gst_rtp_base_depayload_push_list (GstRTPBaseDepayload * filter,
   return res;
 }
 
-/* convert the PacketLost event form a jitterbuffer to a GAP event.
+/* convert the PacketLost event from a jitterbuffer to a GAP event.
  * subclasses can override this.  */
 static gboolean
 gst_rtp_base_depayload_packet_lost (GstRTPBaseDepayload * filter,
     GstEvent * event)
 {
-  GstClockTime timestamp, duration, position;
+  GstClockTime timestamp, duration;
   GstEvent *sevent;
   const GstStructure *s;
 
@@ -632,12 +864,12 @@ gst_rtp_base_depayload_packet_lost (GstRTPBaseDepayload * filter,
   timestamp = -1;
   duration = -1;
 
-  gst_structure_get_clock_time (s, "timestamp", &timestamp);
-  gst_structure_get_clock_time (s, "duration", &duration);
-
-  position = timestamp;
-  if (duration != -1)
-    position += duration;
+  if (!gst_structure_get_clock_time (s, "timestamp", &timestamp) ||
+      !gst_structure_get_clock_time (s, "duration", &duration)) {
+    GST_ERROR_OBJECT (filter,
+        "Packet loss event without timestamp or duration");
+    return FALSE;
+  }
 
   /* send GAP event */
   sevent = gst_event_new_gap (timestamp, duration);
@@ -665,6 +897,7 @@ gst_rtp_base_depayload_change_state (GstElement * element,
       priv->npt_stop = -1;
       priv->play_speed = 1.0;
       priv->play_scale = 1.0;
+      priv->clock_base = -1;
       priv->next_seqnum = -1;
       priv->negotiated = FALSE;
       priv->discont = FALSE;
@@ -681,6 +914,8 @@ gst_rtp_base_depayload_change_state (GstElement * element,
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
+      gst_caps_replace (&priv->last_caps, NULL);
+      gst_event_replace (&priv->segment_event, NULL);
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
       break;
@@ -690,6 +925,39 @@ gst_rtp_base_depayload_change_state (GstElement * element,
   return ret;
 }
 
+static GstStructure *
+gst_rtp_base_depayload_create_stats (GstRTPBaseDepayload * depayload)
+{
+  GstRTPBaseDepayloadPrivate *priv;
+  GstStructure *s;
+  GstClockTime pts = GST_CLOCK_TIME_NONE, dts = GST_CLOCK_TIME_NONE;
+
+  priv = depayload->priv;
+
+  GST_OBJECT_LOCK (depayload);
+  if (depayload->segment.format != GST_FORMAT_UNDEFINED) {
+    pts = gst_segment_to_running_time (&depayload->segment, GST_FORMAT_TIME,
+        priv->pts);
+    dts = gst_segment_to_running_time (&depayload->segment, GST_FORMAT_TIME,
+        priv->dts);
+  }
+  GST_OBJECT_UNLOCK (depayload);
+
+  s = gst_structure_new ("application/x-rtp-depayload-stats",
+      "clock_rate", G_TYPE_UINT, depayload->clock_rate,
+      "npt-start", G_TYPE_UINT64, priv->npt_start,
+      "npt-stop", G_TYPE_UINT64, priv->npt_stop,
+      "play-speed", G_TYPE_DOUBLE, priv->play_speed,
+      "play-scale", G_TYPE_DOUBLE, priv->play_scale,
+      "running-time-dts", G_TYPE_UINT64, dts,
+      "running-time-pts", G_TYPE_UINT64, pts,
+      "seqnum", G_TYPE_UINT, (guint) priv->last_seqnum,
+      "timestamp", G_TYPE_UINT, (guint) priv->last_rtptime, NULL);
+
+  return s;
+}
+
+
 static void
 gst_rtp_base_depayload_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -705,7 +973,15 @@ static void
 gst_rtp_base_depayload_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec)
 {
+  GstRTPBaseDepayload *depayload;
+
+  depayload = GST_RTP_BASE_DEPAYLOAD (object);
+
   switch (prop_id) {
+    case PROP_STATS:
+      g_value_take_boxed (value,
+          gst_rtp_base_depayload_create_stats (depayload));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
