@@ -83,8 +83,13 @@ G_BEGIN_DECLS
  * GST_VIDEO_ENCODER_FLOW_DROPPED:
  *
  * Returned when the event/buffer should be dropped.
+ *
+ * Deprecated: since 1.8. use gst_video_encoder_finish_frame with
+ * a %NULL frame->output_buffer to drop the frame instead.
  */
+#ifndef GST_DISABLE_DEPRECATED
 #define GST_VIDEO_ENCODER_FLOW_DROPPED GST_FLOW_CUSTOM_SUCCESS_1
+#endif
 
 /**
  * GST_VIDEO_ENCODER_INPUT_SEGMENT:
@@ -214,6 +219,21 @@ struct _GstVideoEncoder
  * @flush:              Optional.
  *                      Flush all remaining data from the encoder without
  *                      pushing it downstream. Since: 1.2
+ * @sink_query:     Optional.
+ *                  Query handler on the sink pad. This function should
+ *                  return TRUE if the query could be performed. Subclasses
+ *                  should chain up to the parent implementation to invoke the
+ *                  default handler. Since 1.4
+ * @src_query:      Optional.
+ *                  Query handler on the source pad. This function should
+ *                  return TRUE if the query could be performed. Subclasses
+ *                  should chain up to the parent implementation to invoke the
+ *                  default handler. Since 1.4
+ * @transform_meta: Optional. Transform the metadata on the input buffer to the
+ *                  output buffer. By default this method is copies all meta without
+ *                  tags and meta with only the "video" tag. subclasses can
+ *                  implement this method and return %TRUE if the metadata is to be
+ *                  copied. Since 1.6
  *
  * Subclasses can override any of the available virtual methods or not, as
  * needed. At minimum @handle_frame needs to be overridden, and @set_format
@@ -265,8 +285,18 @@ struct _GstVideoEncoderClass
                                        GstQuery * query);
   gboolean      (*flush)              (GstVideoEncoder *encoder);
 
+  gboolean      (*sink_query)     (GstVideoEncoder *encoder,
+				   GstQuery *query);
+
+  gboolean      (*src_query)      (GstVideoEncoder *encoder,
+				   GstQuery *query);
+
+  gboolean      (*transform_meta) (GstVideoEncoder *encoder,
+                                   GstVideoCodecFrame *frame,
+                                   GstMeta * meta);
+
   /*< private >*/
-  gpointer       _gst_reserved[GST_PADDING_LARGE-1];
+  gpointer       _gst_reserved[GST_PADDING_LARGE-4];
 };
 
 GType                gst_video_encoder_get_type (void);
@@ -298,8 +328,6 @@ GstFlowReturn        gst_video_encoder_finish_frame (GstVideoEncoder *encoder,
 GstCaps *            gst_video_encoder_proxy_getcaps (GstVideoEncoder * enc,
 						      GstCaps         * caps,
                                                       GstCaps         * filter);
-void                 gst_video_encoder_set_discont (GstVideoEncoder *encoder);
-gboolean             gst_video_encoder_get_discont (GstVideoEncoder *encoder);
 
 void                 gst_video_encoder_set_latency (GstVideoEncoder *encoder,
 						    GstClockTime min_latency,
@@ -319,6 +347,12 @@ void                 gst_video_encoder_get_allocator (GstVideoEncoder *encoder,
                                                       GstAllocator **allocator,
                                                       GstAllocationParams *params);
 
+void                 gst_video_encoder_set_min_pts(GstVideoEncoder *encoder, GstClockTime min_pts);
+
+#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstVideoEncoder, gst_object_unref)
+#endif
+
 G_END_DECLS
 
 #endif
