@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -26,18 +26,18 @@
 GST_DEBUG_CATEGORY_STATIC (stream_synchronizer_debug);
 #define GST_CAT_DEFAULT stream_synchronizer_debug
 
-#define GST_STREAM_SYNCHRONIZER_LOCK(obj) G_STMT_START {                   \
-    GST_LOG_OBJECT (obj,                                                \
+#define GST_STREAM_SYNCHRONIZER_LOCK(obj) G_STMT_START {                \
+    GST_TRACE_OBJECT (obj,                                              \
                     "locking from thread %p",                           \
                     g_thread_self ());                                  \
-    g_mutex_lock (&GST_STREAM_SYNCHRONIZER_CAST(obj)->lock);                \
-    GST_LOG_OBJECT (obj,                                                \
+    g_mutex_lock (&GST_STREAM_SYNCHRONIZER_CAST(obj)->lock);            \
+    GST_TRACE_OBJECT (obj,                                              \
                     "locked from thread %p",                            \
                     g_thread_self ());                                  \
 } G_STMT_END
 
-#define GST_STREAM_SYNCHRONIZER_UNLOCK(obj) G_STMT_START {                 \
-    GST_LOG_OBJECT (obj,                                                \
+#define GST_STREAM_SYNCHRONIZER_UNLOCK(obj) G_STMT_START {              \
+    GST_TRACE_OBJECT (obj,                                              \
                     "unlocking from thread %p",                         \
                     g_thread_self ());                                  \
     g_mutex_unlock (&GST_STREAM_SYNCHRONIZER_CAST(obj)->lock);              \
@@ -65,10 +65,13 @@ typedef struct
   GstSegment segment;
 
   gboolean wait;                /* TRUE if waiting/blocking */
-  gboolean new_stream;
-  gboolean drop_discont;
   gboolean is_eos;              /* TRUE if EOS was received */
+  gboolean eos_sent;            /* when EOS was sent downstream */
+  gboolean flushing;            /* set after flush-start and before flush-stop */
   gboolean seen_data;
+  GstClockTime gap_duration;
+
+  GstStreamFlags flags;
 
   GCond stream_finish_cond;
 
@@ -76,11 +79,12 @@ typedef struct
    * default: G_MAXUINT32 */
   guint32 stream_start_seqnum;
   guint32 segment_seqnum;
-} GstStream;
+  guint group_id;
+} GstSyncStream;
 
 /* Must be called with lock! */
 static inline GstPad *
-gst_stream_get_other_pad (GstStream * stream, GstPad * pad)
+gst_stream_get_other_pad (GstSyncStream * stream, GstPad * pad)
 {
   if (stream->sinkpad == pad)
     return gst_object_ref (stream->srcpad);
@@ -93,7 +97,7 @@ gst_stream_get_other_pad (GstStream * stream, GstPad * pad)
 static GstPad *
 gst_stream_get_other_pad_from_pad (GstStreamSynchronizer * self, GstPad * pad)
 {
-  GstStream *stream;
+  GstSyncStream *stream;
   GstPad *opad = NULL;
 
   GST_STREAM_SYNCHRONIZER_LOCK (self);
@@ -135,32 +139,12 @@ gst_stream_synchronizer_iterate_internal_links (GstPad * pad,
   return it;
 }
 
-static gboolean
-gst_stream_synchronizer_query (GstPad * pad, GstObject * parent,
-    GstQuery * query)
-{
-  GstPad *opad;
-  gboolean ret = FALSE;
-
-  GST_LOG_OBJECT (pad, "Handling query %s", GST_QUERY_TYPE_NAME (query));
-
-  opad =
-      gst_stream_get_other_pad_from_pad (GST_STREAM_SYNCHRONIZER (parent), pad);
-  if (opad) {
-    ret = gst_pad_peer_query (opad, query);
-    gst_object_unref (opad);
-  }
-
-  return ret;
-}
-
 /* srcpad functions */
 static gboolean
 gst_stream_synchronizer_src_event (GstPad * pad, GstObject * parent,
     GstEvent * event)
 {
   GstStreamSynchronizer *self = GST_STREAM_SYNCHRONIZER (parent);
-  GstPad *opad;
   gboolean ret = FALSE;
 
   GST_LOG_OBJECT (pad, "Handling event %s: %" GST_PTR_FORMAT,
@@ -172,7 +156,7 @@ gst_stream_synchronizer_src_event (GstPad * pad, GstObject * parent,
       GstClockTimeDiff diff;
       GstClockTime timestamp;
       gint64 running_time_diff = -1;
-      GstStream *stream;
+      GstSyncStream *stream;
 
       gst_event_parse_qos (event, NULL, &proportion, &diff, &timestamp);
       gst_event_unref (event);
@@ -216,23 +200,76 @@ gst_stream_synchronizer_src_event (GstPad * pad, GstObject * parent,
       break;
   }
 
-  opad = gst_stream_get_other_pad_from_pad (self, pad);
-  if (opad) {
-    ret = gst_pad_push_event (opad, event);
-    gst_object_unref (opad);
-  }
+  ret = gst_pad_event_default (pad, parent, event);
 
 out:
   return ret;
 }
 
+/* must be called with the STREAM_SYNCHRONIZER_LOCK */
+static gboolean
+gst_stream_synchronizer_wait (GstStreamSynchronizer * self, GstPad * pad)
+{
+  gboolean ret = FALSE;
+  GstSyncStream *stream;
+
+  while (!self->eos && !self->flushing) {
+    stream = gst_pad_get_element_private (pad);
+    if (!stream) {
+      GST_WARNING_OBJECT (pad, "unknown stream");
+      return ret;
+    }
+    if (stream->flushing) {
+      GST_DEBUG_OBJECT (pad, "Flushing");
+      break;
+    }
+    if (!stream->wait) {
+      GST_DEBUG_OBJECT (pad, "Stream not waiting anymore");
+      break;
+    }
+
+    if (self->send_gap_event) {
+      GstEvent *event;
+
+      if (!GST_CLOCK_TIME_IS_VALID (stream->segment.position)) {
+        GST_WARNING_OBJECT (pad, "Have no position and can't send GAP event");
+        self->send_gap_event = FALSE;
+        continue;
+      }
+
+      event =
+          gst_event_new_gap (stream->segment.position, stream->gap_duration);
+      GST_DEBUG_OBJECT (pad,
+          "Send GAP event, position: %" GST_TIME_FORMAT " duration: %"
+          GST_TIME_FORMAT, GST_TIME_ARGS (stream->segment.position),
+          GST_TIME_ARGS (stream->gap_duration));
+
+      /* drop lock when sending GAP event, which may block in e.g. preroll */
+      GST_STREAM_SYNCHRONIZER_UNLOCK (self);
+      ret = gst_pad_push_event (pad, event);
+      GST_STREAM_SYNCHRONIZER_LOCK (self);
+      if (!ret) {
+        return ret;
+      }
+      self->send_gap_event = FALSE;
+
+      /* force a check on the loop conditions as we unlocked a
+       * few lines above and those variables could have changed */
+      continue;
+    }
+
+    g_cond_wait (&stream->stream_finish_cond, &self->lock);
+  }
+
+  return TRUE;
+}
+
 /* sinkpad functions */
 static gboolean
 gst_stream_synchronizer_sink_event (GstPad * pad, GstObject * parent,
     GstEvent * event)
 {
   GstStreamSynchronizer *self = GST_STREAM_SYNCHRONIZER (parent);
-  GstPad *opad;
   gboolean ret = FALSE;
 
   GST_LOG_OBJECT (pad, "Handling event %s: %" GST_PTR_FORMAT,
@@ -241,35 +278,95 @@ gst_stream_synchronizer_sink_event (GstPad * pad, GstObject * parent,
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_STREAM_START:
     {
-      GstStream *stream;
+      GstSyncStream *stream, *ostream;
       guint32 seqnum = gst_event_get_seqnum (event);
+      guint group_id;
+      gboolean have_group_id;
       GList *l;
       gboolean all_wait = TRUE;
+      gboolean new_stream = TRUE;
+
+      have_group_id = gst_event_parse_group_id (event, &group_id);
 
       GST_STREAM_SYNCHRONIZER_LOCK (self);
+      self->have_group_id &= have_group_id;
+      have_group_id = self->have_group_id;
+
       stream = gst_pad_get_element_private (pad);
-      if (stream && stream->stream_start_seqnum != seqnum) {
+
+      if (!stream) {
+        GST_DEBUG_OBJECT (self, "No stream or STREAM_START from same source");
+        GST_STREAM_SYNCHRONIZER_UNLOCK (self);
+        break;
+      }
+
+      gst_event_parse_stream_flags (event, &stream->flags);
+
+      if ((have_group_id && stream->group_id != group_id) || (!have_group_id
+              && stream->stream_start_seqnum != seqnum)) {
+        stream->is_eos = FALSE;
+        stream->eos_sent = FALSE;
+        stream->flushing = FALSE;
+        stream->stream_start_seqnum = seqnum;
+        stream->group_id = group_id;
+
+        if (!have_group_id) {
+          /* Check if this belongs to a stream that is already there,
+           * e.g. we got the visualizations for an audio stream */
+          for (l = self->streams; l; l = l->next) {
+            ostream = l->data;
+
+            if (ostream != stream && ostream->stream_start_seqnum == seqnum
+                && !ostream->wait) {
+              new_stream = FALSE;
+              break;
+            }
+          }
+
+          if (!new_stream) {
+            GST_DEBUG_OBJECT (pad,
+                "Stream %d belongs to running stream %d, no waiting",
+                stream->stream_number, ostream->stream_number);
+            stream->wait = FALSE;
+
+            GST_STREAM_SYNCHRONIZER_UNLOCK (self);
+            break;
+          }
+        } else if (group_id == self->group_id) {
+          GST_DEBUG_OBJECT (pad, "Stream %d belongs to running group %d, "
+              "no waiting", stream->stream_number, group_id);
+          GST_STREAM_SYNCHRONIZER_UNLOCK (self);
+          break;
+        }
 
         GST_DEBUG_OBJECT (pad, "Stream %d changed", stream->stream_number);
 
-        stream->is_eos = FALSE;
         stream->wait = TRUE;
-        stream->new_stream = TRUE;
 
         for (l = self->streams; l; l = l->next) {
-          GstStream *ostream = l->data;
+          GstSyncStream *ostream = l->data;
 
-          all_wait = all_wait && ostream->wait;
+          all_wait = all_wait && ((ostream->flags & GST_STREAM_FLAG_SPARSE)
+              || (ostream->wait && (!have_group_id
+                      || ostream->group_id == group_id)));
           if (!all_wait)
             break;
         }
+
         if (all_wait) {
           gint64 position = 0;
 
-          GST_DEBUG_OBJECT (self, "All streams have changed -- unblocking");
+          if (have_group_id)
+            GST_DEBUG_OBJECT (self,
+                "All streams have changed to group id %u -- unblocking",
+                group_id);
+          else
+            GST_DEBUG_OBJECT (self, "All streams have changed -- unblocking");
+
+          self->group_id = group_id;
 
           for (l = self->streams; l; l = l->next) {
-            GstStream *ostream = l->data;
+            GstSyncStream *ostream = l->data;
             gint64 stop_running_time;
             gint64 position_running_time;
 
@@ -282,44 +379,43 @@ gst_stream_synchronizer_sink_event (GstPad * pad, GstObject * parent,
               position_running_time =
                   gst_segment_to_running_time (&ostream->segment,
                   GST_FORMAT_TIME, ostream->segment.position);
-              position =
-                  MAX (position, MAX (stop_running_time,
-                      position_running_time));
+
+              position_running_time =
+                  MAX (position_running_time, stop_running_time);
+              position_running_time -=
+                  gst_segment_to_running_time (&ostream->segment,
+                  GST_FORMAT_TIME, ostream->segment.start);
+              position_running_time = MAX (0, position_running_time);
+
+              position = MAX (position, position_running_time);
             }
           }
-          position = MAX (0, position);
-          self->group_start_time = MAX (self->group_start_time, position);
+
+          self->group_start_time += position;
 
           GST_DEBUG_OBJECT (self, "New group start time: %" GST_TIME_FORMAT,
               GST_TIME_ARGS (self->group_start_time));
 
           for (l = self->streams; l; l = l->next) {
-            GstStream *ostream = l->data;
+            GstSyncStream *ostream = l->data;
+            ostream->wait = FALSE;
             g_cond_broadcast (&ostream->stream_finish_cond);
           }
         }
-      } else
-        GST_DEBUG_OBJECT (self, "No stream or STREAM_START from same source");
+      }
+
       GST_STREAM_SYNCHRONIZER_UNLOCK (self);
-    }
       break;
+    }
     case GST_EVENT_SEGMENT:{
-      GstStream *stream;
+      GstSyncStream *stream;
       GstSegment segment;
 
       gst_event_copy_segment (event, &segment);
 
       GST_STREAM_SYNCHRONIZER_LOCK (self);
-      stream = gst_pad_get_element_private (pad);
-      if (stream) {
-        if (stream->wait) {
-          GST_DEBUG_OBJECT (pad, "Stream %d is waiting", stream->stream_number);
-          g_cond_wait (&stream->stream_finish_cond, &self->lock);
-          stream = gst_pad_get_element_private (pad);
-          if (stream)
-            stream->wait = FALSE;
-        }
-      }
+
+      gst_stream_synchronizer_wait (self, pad);
 
       if (self->shutdown) {
         GST_STREAM_SYNCHRONIZER_UNLOCK (self);
@@ -327,12 +423,13 @@ gst_stream_synchronizer_sink_event (GstPad * pad, GstObject * parent,
         goto done;
       }
 
+      stream = gst_pad_get_element_private (pad);
       if (stream && segment.format == GST_FORMAT_TIME) {
-        if (stream->new_stream) {
-          stream->new_stream = FALSE;
-          stream->drop_discont = TRUE;
-          segment.base = self->group_start_time;
-        }
+        GST_DEBUG_OBJECT (pad,
+            "New stream, updating base from %" GST_TIME_FORMAT " to %"
+            GST_TIME_FORMAT, GST_TIME_ARGS (segment.base),
+            GST_TIME_ARGS (segment.base + self->group_start_time));
+        segment.base += self->group_start_time;
 
         GST_DEBUG_OBJECT (pad, "Segment was: %" GST_SEGMENT_FORMAT,
             &stream->segment);
@@ -351,32 +448,32 @@ gst_stream_synchronizer_sink_event (GstPad * pad, GstObject * parent,
           gst_event_unref (event);
           event = tmpev;
         }
-
       } else if (stream) {
         GST_WARNING_OBJECT (pad, "Non-TIME segment: %s",
             gst_format_get_name (segment.format));
         gst_segment_init (&stream->segment, GST_FORMAT_UNDEFINED);
-        /* Since this stream is not time-based, we mark it so that
-         * other streams don't wait forever on it */
-        stream->wait = TRUE;
       }
       GST_STREAM_SYNCHRONIZER_UNLOCK (self);
       break;
     }
     case GST_EVENT_FLUSH_START:{
-      GstStream *stream;
+      GstSyncStream *stream;
 
       GST_STREAM_SYNCHRONIZER_LOCK (self);
       stream = gst_pad_get_element_private (pad);
+      self->eos = FALSE;
       if (stream) {
         GST_DEBUG_OBJECT (pad, "Flushing streams");
+        stream->flushing = TRUE;
         g_cond_broadcast (&stream->stream_finish_cond);
       }
       GST_STREAM_SYNCHRONIZER_UNLOCK (self);
       break;
     }
     case GST_EVENT_FLUSH_STOP:{
-      GstStream *stream;
+      GstSyncStream *stream;
+      GList *l;
+      GstClockTime new_group_start_time = 0;
 
       GST_STREAM_SYNCHRONIZER_LOCK (self);
       stream = gst_pad_get_element_private (pad);
@@ -386,22 +483,63 @@ gst_stream_synchronizer_sink_event (GstPad * pad, GstObject * parent,
         gst_segment_init (&stream->segment, GST_FORMAT_UNDEFINED);
 
         stream->is_eos = FALSE;
+        stream->eos_sent = FALSE;
+        stream->flushing = FALSE;
         stream->wait = FALSE;
-        stream->new_stream = FALSE;
-        stream->drop_discont = FALSE;
-        stream->seen_data = FALSE;
         g_cond_broadcast (&stream->stream_finish_cond);
       }
+
+      for (l = self->streams; l; l = l->next) {
+        GstSyncStream *ostream = l->data;
+        GstClockTime start_running_time;
+
+        if (ostream == stream || ostream->flushing)
+          continue;
+
+        if (ostream->segment.format == GST_FORMAT_TIME) {
+          start_running_time =
+              gst_segment_to_running_time (&ostream->segment,
+              GST_FORMAT_TIME, ostream->segment.start);
+
+          new_group_start_time = MAX (new_group_start_time, start_running_time);
+        }
+      }
+
+      GST_DEBUG_OBJECT (pad,
+          "Updating group start time from %" GST_TIME_FORMAT " to %"
+          GST_TIME_FORMAT, GST_TIME_ARGS (self->group_start_time),
+          GST_TIME_ARGS (new_group_start_time));
+      self->group_start_time = new_group_start_time;
       GST_STREAM_SYNCHRONIZER_UNLOCK (self);
       break;
+    }
+      /* unblocking EOS wait when track switch. */
+    case GST_EVENT_CUSTOM_DOWNSTREAM_OOB:{
+      if (gst_event_has_name (event, "playsink-custom-video-flush")
+          || gst_event_has_name (event, "playsink-custom-audio-flush")
+          || gst_event_has_name (event, "playsink-custom-subtitle-flush")) {
+        GstSyncStream *stream;
+
+        GST_STREAM_SYNCHRONIZER_LOCK (self);
+        stream = gst_pad_get_element_private (pad);
+        if (stream) {
+          stream->is_eos = FALSE;
+          stream->eos_sent = FALSE;
+          stream->wait = FALSE;
+          g_cond_broadcast (&stream->stream_finish_cond);
+        }
+        GST_STREAM_SYNCHRONIZER_UNLOCK (self);
+      }
+      break;
     }
     case GST_EVENT_EOS:{
-      GstStream *stream;
+      GstSyncStream *stream;
       GList *l;
       gboolean all_eos = TRUE;
       gboolean seen_data;
       GSList *pads = NULL;
       GstPad *srcpad;
+      GstClockTime timestamp;
 
       GST_STREAM_SYNCHRONIZER_LOCK (self);
       stream = gst_pad_get_element_private (pad);
@@ -417,8 +555,17 @@ gst_stream_synchronizer_sink_event (GstPad * pad, GstObject * parent,
       seen_data = stream->seen_data;
       srcpad = gst_object_ref (stream->srcpad);
 
+      if (seen_data && stream->segment.position != -1)
+        timestamp = stream->segment.position;
+      else if (stream->segment.rate < 0.0 || stream->segment.stop == -1)
+        timestamp = stream->segment.start;
+      else
+        timestamp = stream->segment.stop;
+
+      stream->segment.position = timestamp;
+
       for (l = self->streams; l; l = l->next) {
-        GstStream *ostream = l->data;
+        GstSyncStream *ostream = l->data;
 
         all_eos = all_eos && ostream->is_eos;
         if (!all_eos)
@@ -427,52 +574,69 @@ gst_stream_synchronizer_sink_event (GstPad * pad, GstObject * parent,
 
       if (all_eos) {
         GST_DEBUG_OBJECT (self, "All streams are EOS -- forwarding");
+        self->eos = TRUE;
         for (l = self->streams; l; l = l->next) {
-          GstStream *ostream = l->data;
+          GstSyncStream *ostream = l->data;
           /* local snapshot of current pads */
           gst_object_ref (ostream->srcpad);
           pads = g_slist_prepend (pads, ostream->srcpad);
         }
       }
-      GST_STREAM_SYNCHRONIZER_UNLOCK (self);
-      /* drop lock when sending eos, which may block in e.g. preroll */
       if (pads) {
         GstPad *pad;
         GSList *epad;
+        GstSyncStream *ostream;
 
         ret = TRUE;
         epad = pads;
         while (epad) {
           pad = epad->data;
-          GST_DEBUG_OBJECT (pad, "Pushing EOS");
-          ret = ret && gst_pad_push_event (pad, gst_event_new_eos ());
+          ostream = gst_pad_get_element_private (pad);
+          if (ostream) {
+            g_cond_broadcast (&ostream->stream_finish_cond);
+          }
+
           gst_object_unref (pad);
           epad = g_slist_next (epad);
         }
         g_slist_free (pads);
       } else {
-        /* if EOS, but no data has passed, then send something to replace EOS
-         * for preroll purposes */
-        if (!seen_data) {
-          GstEvent *gap_event;
-
-          gap_event = gst_event_new_gap (0, 0);
-          gst_pad_push_event (srcpad, gap_event);
+        if (seen_data) {
+          self->send_gap_event = TRUE;
+          stream->gap_duration = GST_CLOCK_TIME_NONE;
+          stream->wait = TRUE;
+          ret = gst_stream_synchronizer_wait (self, srcpad);
         }
       }
+
+      /* send eos if haven't seen data. seen_data will be true if data buffer
+       * of the track have received in anytime. sink is ready if seen_data is
+       * true, so can send GAP event. Will send EOS if sink isn't ready. The
+       * scenario for the case is one track haven't any media data and then
+       * send EOS. Or no any valid media data in one track, so decoder can't
+       * get valid CAPS for the track. sink can't ready without received CAPS.*/
+      if (!seen_data || self->eos) {
+        GST_DEBUG_OBJECT (pad, "send EOS event");
+        /* drop lock when sending eos, which may block in e.g. preroll */
+        GST_STREAM_SYNCHRONIZER_UNLOCK (self);
+        ret = gst_pad_push_event (srcpad, gst_event_new_eos ());
+        GST_STREAM_SYNCHRONIZER_LOCK (self);
+        stream = gst_pad_get_element_private (pad);
+        if (stream) {
+          stream->eos_sent = TRUE;
+        }
+      }
+
       gst_object_unref (srcpad);
       gst_event_unref (event);
+      GST_STREAM_SYNCHRONIZER_UNLOCK (self);
       goto done;
     }
     default:
       break;
   }
 
-  opad = gst_stream_get_other_pad_from_pad (self, pad);
-  if (opad) {
-    ret = gst_pad_push_event (opad, event);
-    gst_object_unref (opad);
-  }
+  ret = gst_pad_event_default (pad, parent, event);
 
 done:
 
@@ -486,7 +650,8 @@ gst_stream_synchronizer_sink_chain (GstPad * pad, GstObject * parent,
   GstStreamSynchronizer *self = GST_STREAM_SYNCHRONIZER (parent);
   GstPad *opad;
   GstFlowReturn ret = GST_FLOW_ERROR;
-  GstStream *stream;
+  GstSyncStream *stream;
+  GstClockTime duration = GST_CLOCK_TIME_NONE;
   GstClockTime timestamp = GST_CLOCK_TIME_NONE;
   GstClockTime timestamp_end = GST_CLOCK_TIME_NONE;
 
@@ -499,30 +664,25 @@ gst_stream_synchronizer_sink_chain (GstPad * pad, GstObject * parent,
       GST_BUFFER_OFFSET (buffer), GST_BUFFER_OFFSET_END (buffer));
 
   timestamp = GST_BUFFER_TIMESTAMP (buffer);
-  if (GST_BUFFER_TIMESTAMP_IS_VALID (buffer)
-      && GST_BUFFER_DURATION_IS_VALID (buffer))
-    timestamp_end = timestamp + GST_BUFFER_DURATION (buffer);
+  duration = GST_BUFFER_DURATION (buffer);
+  if (GST_CLOCK_TIME_IS_VALID (timestamp)
+      && GST_CLOCK_TIME_IS_VALID (duration))
+    timestamp_end = timestamp + duration;
 
   GST_STREAM_SYNCHRONIZER_LOCK (self);
   stream = gst_pad_get_element_private (pad);
 
   if (stream) {
     stream->seen_data = TRUE;
-    if (stream->drop_discont) {
-      if (GST_BUFFER_IS_DISCONT (buffer)) {
-        GST_DEBUG_OBJECT (pad, "removing DISCONT from buffer %p", buffer);
-        buffer = gst_buffer_make_writable (buffer);
-        GST_BUFFER_FLAG_UNSET (buffer, GST_BUFFER_FLAG_DISCONT);
-      }
-      stream->drop_discont = FALSE;
-    }
-
     if (stream->segment.format == GST_FORMAT_TIME
         && GST_CLOCK_TIME_IS_VALID (timestamp)) {
       GST_LOG_OBJECT (pad,
           "Updating position from %" GST_TIME_FORMAT " to %" GST_TIME_FORMAT,
           GST_TIME_ARGS (stream->segment.position), GST_TIME_ARGS (timestamp));
-      stream->segment.position = timestamp;
+      if (stream->segment.rate > 0.0)
+        stream->segment.position = timestamp;
+      else
+        stream->segment.position = timestamp_end;
     }
   }
   GST_STREAM_SYNCHRONIZER_UNLOCK (self);
@@ -539,22 +699,35 @@ gst_stream_synchronizer_sink_chain (GstPad * pad, GstObject * parent,
 
     GST_STREAM_SYNCHRONIZER_LOCK (self);
     stream = gst_pad_get_element_private (pad);
-    if (stream && stream->segment.format == GST_FORMAT_TIME
-        && GST_CLOCK_TIME_IS_VALID (timestamp_end)) {
-      GST_LOG_OBJECT (pad,
-          "Updating position from %" GST_TIME_FORMAT " to %" GST_TIME_FORMAT,
-          GST_TIME_ARGS (stream->segment.position),
-          GST_TIME_ARGS (timestamp_end));
-      stream->segment.position = timestamp_end;
+    if (stream && stream->segment.format == GST_FORMAT_TIME) {
+      GstClockTime position;
+
+      if (stream->segment.rate > 0.0)
+        position = timestamp_end;
+      else
+        position = timestamp;
+
+      if (GST_CLOCK_TIME_IS_VALID (position)) {
+        GST_LOG_OBJECT (pad,
+            "Updating position from %" GST_TIME_FORMAT " to %" GST_TIME_FORMAT,
+            GST_TIME_ARGS (stream->segment.position), GST_TIME_ARGS (position));
+        stream->segment.position = position;
+      }
     }
 
     /* Advance EOS streams if necessary. For non-EOS
      * streams the demuxers should already do this! */
+    if (!GST_CLOCK_TIME_IS_VALID (timestamp_end) &&
+        GST_CLOCK_TIME_IS_VALID (timestamp)) {
+      timestamp_end = timestamp + GST_SECOND;
+    }
+
     for (l = self->streams; l; l = l->next) {
-      GstStream *ostream = l->data;
+      GstSyncStream *ostream = l->data;
       gint64 position;
 
-      if (!ostream->is_eos || ostream->segment.format != GST_FORMAT_TIME)
+      if (!ostream->is_eos || ostream->eos_sent ||
+          ostream->segment.format != GST_FORMAT_TIME)
         continue;
 
       if (ostream->segment.position != -1)
@@ -563,27 +736,22 @@ gst_stream_synchronizer_sink_chain (GstPad * pad, GstObject * parent,
         position = ostream->segment.start;
 
       /* Is there a 1 second lag? */
-      if (position != -1 && position + GST_SECOND < timestamp_end) {
-        gint64 new_start, new_stop;
+      if (position != -1 && GST_CLOCK_TIME_IS_VALID (timestamp_end) &&
+          position + GST_SECOND < timestamp_end) {
+        gint64 new_start;
 
         new_start = timestamp_end - GST_SECOND;
-        if (ostream->segment.stop == -1)
-          new_stop = -1;
-        else
-          new_stop = MAX (new_start, ostream->segment.stop);
 
         GST_DEBUG_OBJECT (ostream->sinkpad,
             "Advancing stream %u from %" GST_TIME_FORMAT " to %"
             GST_TIME_FORMAT, ostream->stream_number, GST_TIME_ARGS (position),
             GST_TIME_ARGS (new_start));
 
-        ostream->segment.start = new_start;
-        ostream->segment.stop = new_stop;
-        ostream->segment.time = new_start;
         ostream->segment.position = new_start;
 
-        gst_pad_push_event (ostream->srcpad,
-            gst_event_new_segment (&ostream->segment));
+        self->send_gap_event = TRUE;
+        ostream->gap_duration = new_start - position;
+        g_cond_broadcast (&ostream->stream_finish_cond);
       }
     }
     GST_STREAM_SYNCHRONIZER_UNLOCK (self);
@@ -598,19 +766,21 @@ gst_stream_synchronizer_request_new_pad (GstElement * element,
     GstPadTemplate * temp, const gchar * name, const GstCaps * caps)
 {
   GstStreamSynchronizer *self = GST_STREAM_SYNCHRONIZER (element);
-  GstStream *stream;
+  GstSyncStream *stream;
   gchar *tmp;
 
   GST_STREAM_SYNCHRONIZER_LOCK (self);
   GST_DEBUG_OBJECT (self, "Requesting new pad for stream %d",
       self->current_stream_number);
 
-  stream = g_slice_new0 (GstStream);
+  stream = g_slice_new0 (GstSyncStream);
   stream->transform = self;
   stream->stream_number = self->current_stream_number;
   g_cond_init (&stream->stream_finish_cond);
   stream->stream_start_seqnum = G_MAXUINT32;
   stream->segment_seqnum = G_MAXUINT32;
+  stream->group_id = G_MAXUINT;
+  stream->seen_data = FALSE;
 
   tmp = g_strdup_printf ("sink_%u", self->current_stream_number);
   stream->sinkpad = gst_pad_new_from_static_template (&sinktemplate, tmp);
@@ -618,12 +788,13 @@ gst_stream_synchronizer_request_new_pad (GstElement * element,
   gst_pad_set_element_private (stream->sinkpad, stream);
   gst_pad_set_iterate_internal_links_function (stream->sinkpad,
       GST_DEBUG_FUNCPTR (gst_stream_synchronizer_iterate_internal_links));
-  gst_pad_set_query_function (stream->sinkpad,
-      GST_DEBUG_FUNCPTR (gst_stream_synchronizer_query));
   gst_pad_set_event_function (stream->sinkpad,
       GST_DEBUG_FUNCPTR (gst_stream_synchronizer_sink_event));
   gst_pad_set_chain_function (stream->sinkpad,
       GST_DEBUG_FUNCPTR (gst_stream_synchronizer_sink_chain));
+  GST_PAD_SET_PROXY_CAPS (stream->sinkpad);
+  GST_PAD_SET_PROXY_ALLOCATION (stream->sinkpad);
+  GST_PAD_SET_PROXY_SCHEDULING (stream->sinkpad);
 
   tmp = g_strdup_printf ("src_%u", self->current_stream_number);
   stream->srcpad = gst_pad_new_from_static_template (&srctemplate, tmp);
@@ -631,10 +802,11 @@ gst_stream_synchronizer_request_new_pad (GstElement * element,
   gst_pad_set_element_private (stream->srcpad, stream);
   gst_pad_set_iterate_internal_links_function (stream->srcpad,
       GST_DEBUG_FUNCPTR (gst_stream_synchronizer_iterate_internal_links));
-  gst_pad_set_query_function (stream->srcpad,
-      GST_DEBUG_FUNCPTR (gst_stream_synchronizer_query));
   gst_pad_set_event_function (stream->srcpad,
       GST_DEBUG_FUNCPTR (gst_stream_synchronizer_src_event));
+  GST_PAD_SET_PROXY_CAPS (stream->srcpad);
+  GST_PAD_SET_PROXY_ALLOCATION (stream->srcpad);
+  GST_PAD_SET_PROXY_SCHEDULING (stream->srcpad);
 
   gst_segment_init (&stream->segment, GST_FORMAT_UNDEFINED);
 
@@ -658,7 +830,7 @@ gst_stream_synchronizer_request_new_pad (GstElement * element,
 /* Must be called with lock! */
 static void
 gst_stream_synchronizer_release_stream (GstStreamSynchronizer * self,
-    GstStream * stream)
+    GstSyncStream * stream)
 {
   GList *l;
 
@@ -671,6 +843,10 @@ gst_stream_synchronizer_release_stream (GstStreamSynchronizer * self,
     }
   }
   g_assert (l != NULL);
+  if (self->streams == NULL) {
+    self->have_group_id = TRUE;
+    self->group_id = G_MAXUINT;
+  }
 
   /* we can drop the lock, since stream exists now only local.
    * Moreover, we should drop, to prevent deadlock with STREAM_LOCK
@@ -684,30 +860,8 @@ gst_stream_synchronizer_release_stream (GstStreamSynchronizer * self,
   gst_pad_set_active (stream->sinkpad, FALSE);
   gst_element_remove_pad (GST_ELEMENT_CAST (self), stream->sinkpad);
 
-  if (stream->segment.format == GST_FORMAT_TIME) {
-    gint64 stop_running_time;
-    gint64 position_running_time;
-
-    stop_running_time =
-        gst_segment_to_running_time (&stream->segment, GST_FORMAT_TIME,
-        stream->segment.stop);
-    position_running_time =
-        gst_segment_to_running_time (&stream->segment, GST_FORMAT_TIME,
-        stream->segment.position);
-    stop_running_time = MAX (stop_running_time, position_running_time);
-
-    if (stop_running_time > self->group_start_time) {
-      GST_DEBUG_OBJECT (stream->sinkpad,
-          "Updating global start running time from %" GST_TIME_FORMAT " to %"
-          GST_TIME_FORMAT, GST_TIME_ARGS (self->group_start_time),
-          GST_TIME_ARGS (stop_running_time));
-
-      self->group_start_time = stop_running_time;
-    }
-  }
-
   g_cond_clear (&stream->stream_finish_cond);
-  g_slice_free (GstStream, stream);
+  g_slice_free (GstSyncStream, stream);
 
   /* NOTE: In theory we have to check here if all streams
    * are EOS but the one that was removed wasn't and then
@@ -726,7 +880,7 @@ static void
 gst_stream_synchronizer_release_pad (GstElement * element, GstPad * pad)
 {
   GstStreamSynchronizer *self = GST_STREAM_SYNCHRONIZER (element);
-  GstStream *stream;
+  GstSyncStream *stream;
 
   GST_STREAM_SYNCHRONIZER_LOCK (self);
   stream = gst_pad_get_element_private (pad);
@@ -749,23 +903,29 @@ gst_stream_synchronizer_change_state (GstElement * element,
     case GST_STATE_CHANGE_NULL_TO_READY:
       GST_DEBUG_OBJECT (self, "State change NULL->READY");
       self->shutdown = FALSE;
+      self->send_gap_event = FALSE;
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
       GST_DEBUG_OBJECT (self, "State change READY->PAUSED");
       self->group_start_time = 0;
+      self->have_group_id = TRUE;
+      self->group_id = G_MAXUINT;
       self->shutdown = FALSE;
+      self->flushing = FALSE;
+      self->eos = FALSE;
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:{
       GList *l;
 
-      GST_DEBUG_OBJECT (self, "State change READY->NULL");
+      GST_DEBUG_OBJECT (self, "State change PAUSED->READY");
 
       GST_STREAM_SYNCHRONIZER_LOCK (self);
+      self->flushing = TRUE;
+      self->shutdown = TRUE;
       for (l = self->streams; l; l = l->next) {
-        GstStream *ostream = l->data;
+        GstSyncStream *ostream = l->data;
         g_cond_broadcast (&ostream->stream_finish_cond);
       }
-      self->shutdown = TRUE;
       GST_STREAM_SYNCHRONIZER_UNLOCK (self);
     }
     default:
@@ -778,6 +938,26 @@ gst_stream_synchronizer_change_state (GstElement * element,
     return ret;
 
   switch (transition) {
+    case GST_STATE_CHANGE_PLAYING_TO_PAUSED:{
+      GList *l;
+
+      GST_DEBUG_OBJECT (self, "State change PLAYING->PAUSED");
+
+      GST_STREAM_SYNCHRONIZER_LOCK (self);
+      for (l = self->streams; l; l = l->next) {
+        GstSyncStream *stream = l->data;
+        /* send GAP event to sink to finished pre-roll. The reason is function
+         * chain () will be blocked on pad_push (), so can't trigger the track
+         * which reach EOS to send GAP event. */
+        if (stream->is_eos && !stream->eos_sent) {
+          self->send_gap_event = TRUE;
+          stream->gap_duration = GST_CLOCK_TIME_NONE;
+          g_cond_broadcast (&stream->stream_finish_cond);
+        }
+      }
+      GST_STREAM_SYNCHRONIZER_UNLOCK (self);
+      break;
+    }
     case GST_STATE_CHANGE_PAUSED_TO_READY:{
       GList *l;
 
@@ -785,14 +965,16 @@ gst_stream_synchronizer_change_state (GstElement * element,
       self->group_start_time = 0;
 
       GST_STREAM_SYNCHRONIZER_LOCK (self);
+      self->send_gap_event = FALSE;
       for (l = self->streams; l; l = l->next) {
-        GstStream *stream = l->data;
+        GstSyncStream *stream = l->data;
 
         gst_segment_init (&stream->segment, GST_FORMAT_UNDEFINED);
+        stream->gap_duration = GST_CLOCK_TIME_NONE;
         stream->wait = FALSE;
-        stream->new_stream = FALSE;
-        stream->drop_discont = FALSE;
         stream->is_eos = FALSE;
+        stream->eos_sent = FALSE;
+        stream->flushing = FALSE;
       }
       GST_STREAM_SYNCHRONIZER_UNLOCK (self);
       break;
@@ -836,9 +1018,6 @@ gst_stream_synchronizer_class_init (GstStreamSynchronizerClass * klass)
   GObjectClass *gobject_class = (GObjectClass *) klass;
   GstElementClass *element_class = (GstElementClass *) klass;
 
-  GST_DEBUG_CATEGORY_INIT (stream_synchronizer_debug,
-      "streamsynchronizer", 0, "Stream Synchronizer");
-
   gobject_class->finalize = gst_stream_synchronizer_finalize;
 
   gst_element_class_add_pad_template (element_class,
@@ -858,3 +1037,13 @@ gst_stream_synchronizer_class_init (GstStreamSynchronizerClass * klass)
   element_class->release_pad =
       GST_DEBUG_FUNCPTR (gst_stream_synchronizer_release_pad);
 }
+
+gboolean
+gst_stream_synchronizer_plugin_init (GstPlugin * plugin)
+{
+  GST_DEBUG_CATEGORY_INIT (stream_synchronizer_debug,
+      "streamsynchronizer", 0, "Stream Synchronizer");
+
+  return gst_element_register (plugin, "streamsynchronizer", GST_RANK_NONE,
+      GST_TYPE_STREAM_SYNCHRONIZER);
+}
