@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifndef __GST_VIDEO_RATE_H__
@@ -65,6 +65,8 @@ struct _GstVideoRate
   guint64 average_period;
   GstClockTimeDiff wanted_diff; /* target average diff */
   GstClockTimeDiff average;     /* moving average period */
+  gboolean force_variable_rate;
+  gboolean updating_caps;
 
   /* segment handling */
   GstSegment segment;
