@@ -1134,9 +1134,9 @@ paint_tmpline_AYUV (paintinfo * p, int x, int w)
 
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN
   value = (p->color->A << 0) | (p->color->Y << 8) |
-      (p->color->U << 16) | (p->color->V << 24);
+      (p->color->U << 16) | ((guint32) p->color->V << 24);
 #else
-  value = (p->color->A << 24) | (p->color->Y << 16) |
+  value = ((guint32) p->color->A << 24) | (p->color->Y << 16) |
       (p->color->U << 8) | (p->color->V << 0);
 #endif
 
@@ -1186,8 +1186,6 @@ convert_hline_generic (paintinfo * p, GstVideoFrame * frame, int y)
 
     for (i = 0; i < n_lines; i++) {
       idx = CLAMP (y + i + offset, 0, height - 1);
-
-      GST_DEBUG ("line %d, %d, idx %d", i, y + i + offset, idx);
       lines[i] = p->lines[idx % n_lines];
     }
 
@@ -1198,7 +1196,6 @@ convert_hline_generic (paintinfo * p, GstVideoFrame * frame, int y)
       idx = y + i + offset;
       if (idx > height - 1)
         break;
-      GST_DEBUG ("pack line %d", idx);
       finfo->pack_func (finfo, GST_VIDEO_PACK_FLAG_NONE,
           lines[i], 0, frame->data, frame->info.stride,
           frame->info.chroma_site, idx, width);
@@ -1336,3 +1333,55 @@ gst_video_test_src_spokes (GstVideoTestSrc * v, GstVideoFrame * frame)
     videotestsrc_convert_tmpline (p, frame, j);
   }
 }
+
+void
+gst_video_test_src_gradient (GstVideoTestSrc * v, GstVideoFrame * frame)
+{
+  int i;
+  int j;
+  paintinfo pi = PAINT_INFO_INIT;
+  paintinfo *p = &pi;
+  struct vts_color_struct color;
+  int w = frame->info.width, h = frame->info.height;
+
+  videotestsrc_setup_paintinfo (v, p, w, h);
+
+  color = p->colors[COLOR_BLACK];
+  p->color = &color;
+
+  for (j = 0; j < h; j++) {
+    int y = j * 255.0 / h;
+    for (i = 0; i < w; i++) {
+      p->tmpline_u8[i] = y;
+    }
+    videotestsrc_blend_line (v, p->tmpline, p->tmpline_u8,
+        &p->foreground_color, &p->background_color, w);
+    videotestsrc_convert_tmpline (p, frame, j);
+  }
+}
+
+void
+gst_video_test_src_colors (GstVideoTestSrc * v, GstVideoFrame * frame)
+{
+  int i;
+  int j;
+  paintinfo pi = PAINT_INFO_INIT;
+  paintinfo *p = &pi;
+  struct vts_color_struct color;
+  int w = frame->info.width, h = frame->info.height;
+
+  videotestsrc_setup_paintinfo (v, p, w, h);
+
+  color = p->colors[COLOR_BLACK];
+  p->color = &color;
+
+  for (j = 0; j < h; j++) {
+    for (i = 0; i < w; i++) {
+      p->tmpline[i * 4 + 0] = 0xff;
+      p->tmpline[i * 4 + 1] = ((i * 4096) / w) % 256;
+      p->tmpline[i * 4 + 2] = (((j * 16) / h) << 4) | ((i * 16) / w);
+      p->tmpline[i * 4 + 3] = ((j * 4096) / h) % 256;
+    }
+    videotestsrc_convert_tmpline (p, frame, j);
+  }
+}
