@@ -105,7 +105,13 @@ static SubParseInputChunk srt_input[] = {
       360 * GST_SECOND, 480 * GST_SECOND, "Rock &amp; Roll"}, {
         "28\n00:10:00,000 --> 00:11:00,000\n"
         "<font \"#0000FF\"><joj>This is </xxx>in blue but <5</font>\n\n",
-      600 * GST_SECOND, 660 * GST_SECOND, "This is in blue but &lt;5"}
+      600 * GST_SECOND, 660 * GST_SECOND, "This is in blue but &lt;5"}, {
+        /* closing tags should be recognised properly even if there's a space */
+        "29\n00:11:00,000 --> 00:12:00,000\n" "<i>italics</ i>\n\n",
+      660 * GST_SECOND, 720 * GST_SECOND, "<i>italics</i>"}, {
+        /* closing tags should be escaped and fixed up if not recognised */
+        "30\n00:12:00,000 --> 00:12:01,000\n" "<i>italics</ x>\n\n",
+      720 * GST_SECOND, 721 * GST_SECOND, "<i>italics&lt;/ x&gt;</i>"},
 };
 
 /* starts with chunk number 0 (not exactly according to spec) */
@@ -771,6 +777,51 @@ GST_START_TEST (test_sami_bad_entities)
 
 GST_END_TEST;
 
+GST_START_TEST (test_sami_comment)
+{
+  SubParseInputChunk sami_input[] = {
+    {"<SAMI>\n"
+          "<!--\n"
+          "=======\n"
+          "foo bar\n"
+          "=======\n"
+          "-->\n"
+          "<BODY>\n"
+          "    <SYNC Start=1000>\n"
+          "        <P Class=\"C====\">\n" "            &nbsp &\n",
+          1000 * GST_MSECOND, 2000 * GST_MSECOND,
+        "\xc2\xa0 &amp;"},
+    {"    <SYNC Start=2000>\n"
+          "        <P Class=CC>\n"
+          "            &#xa0 &#177 &#180;\n" "</BODY>\n" "</SAMI>\n",
+          2000 * GST_MSECOND, GST_CLOCK_TIME_NONE,
+        "\xc2\xa0 \xc2\xb1 \xc2\xb4"}
+  };
+
+  do_test (sami_input, G_N_ELEMENTS (sami_input), "pango-markup");
+}
+
+GST_END_TEST;
+
+GST_START_TEST (test_lrc)
+{
+  SubParseInputChunk lrc_input[] = {
+    {"[ar:123]\n" "[ti:Title]\n" "[al:Album]\n" "[00:02.23]Line 1\n",
+          2230 * GST_MSECOND, GST_CLOCK_TIME_NONE,
+        "Line 1"},
+    {"[00:05.10]Line 2\n",
+          5100 * GST_MSECOND, GST_CLOCK_TIME_NONE,
+        "Line 2"},
+    {"[00:06.123]Line 3\n",
+          6123 * GST_MSECOND, GST_CLOCK_TIME_NONE,
+        "Line 3"}
+  };
+
+  do_test (lrc_input, G_N_ELEMENTS (lrc_input), "utf8");
+}
+
+GST_END_TEST;
+
 /* TODO:
  *  - add/modify tests so that lines aren't dogfed to the parsers in complete
  *    lines or sets of complete lines, but rather in random chunks
@@ -803,6 +854,8 @@ subparse_suite (void)
   tcase_add_test (tc_chain, test_sami_xml_entities);
   tcase_add_test (tc_chain, test_sami_html_entities);
   tcase_add_test (tc_chain, test_sami_bad_entities);
+  tcase_add_test (tc_chain, test_sami_comment);
+  tcase_add_test (tc_chain, test_lrc);
   return s;
 }
 
