@@ -151,7 +151,7 @@ GST_START_TEST (test_memory_stream)
 GST_END_TEST;
 
 static Suite *
-gio_testsuite (void)
+gio_suite (void)
 {
   Suite *s = suite_create ("gio");
   TCase *tc_chain = tcase_create ("general");
@@ -162,19 +162,4 @@ gio_testsuite (void)
   return s;
 }
 
-int
-main (int argc, char **argv)
-{
-  int nf;
-
-  Suite *s = gio_testsuite ();
-  SRunner *sr = srunner_create (s);
-
-  gst_check_init (&argc, &argv);
-
-  srunner_run_all (sr, CK_NORMAL);
-  nf = srunner_ntests_failed (sr);
-  srunner_free (sr);
-
-  return nf;
-}
+GST_CHECK_MAIN (gio);
